/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;

import java.util.Date;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;


/**
 * Controller inn charge of LifeCycle activites for the MIDlet
 */
public class LifeCycleController implements Runnable {
    
    private MIDlet midlet;

    public static boolean isPaused;
    
    private static LifeCycleController lifeCycleController;
    
    private LifeCycleController() {
        midlet = UIController.midlet;
    }
    
    public static LifeCycleController getInstance() {
        if (lifeCycleController==null) {
            lifeCycleController = new LifeCycleController();
        }
        return lifeCycleController;
    }
    
    public void pause() {
        Log.info("Application Paused");
        //save status etc.
        isPaused=true;
    }
    
    public void resume() {
        Log.info("Application Resumed");
        // what happens resuming the UI?
        // take time showing a wait screen
        UIController.updateInboxMessageList(UIController.getInboxMessageList().getFunTitle());
    }
    
    /**
     * This method manages the exit of the MIDlet, doing all the
     * jobs related to the OTA messages and alarms before the actual exit
     */
    public void exit() {
         try {
            AlarmManager alarmManager = AlarmManager.getInstance();
            if (UIController.getConfig().isSchedulerEnabled()) {
                int pollInterval = UIController.getConfig().getPollInterval();
                if (pollInterval >= 1440) {
                    alarmManager.setAlarm(
                        UIController.getConfig().getNextTimeAlarm());
                } else {
                    long millisecs = pollInterval*60000;
                    Date now = new Date();
                    alarmManager.setAlarm(now.getTime() + millisecs);
                }
                alarmManager.scheduleMIDletAlarm();
            }
            /*if (!(UIController.getConfig().isSmsListenerEnabled())) {
                OTAMessagesManager.getInstance().unregisterForSANMessage();
            } else {
                OTAMessagesManager.getInstance().registerForSANMessage();
            }*/
        } catch (SecurityException ex) {
            Log.error("Exit: User denied the authorization for alarm");
        } catch (Exception ex) {
            Log.error("Exit: " + ex.toString());
            ex.printStackTrace();
        } finally {
            try {
                ((Killable) midlet).exit();
            } catch (MIDletStateChangeException ex) {
                Log.error("Exit: " + ex.toString());
                ex.printStackTrace();
            }
        }
    }
    
    public void run() {
        exit();
    }
    
}
