/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.controller;

import com.funambol.mail.Folder;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mail.StoreFactory;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.config.ConfigException;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.syncml.MailSyncSource;
import com.funambol.mailclient.syncml.FlagQueue;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.mailclient.ui.view.InboxMessageList;
import com.funambol.mailclient.ui.view.ModalPopup;
import com.funambol.mailclient.ui.view.PopupAction;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Log;
import com.funambol.util.BasicSyncListener;
import com.funambol.util.MailDateFormatter;
import com.funambol.mailclient.ui.utils.UserNotificationManager;
import com.funambol.push.CTPService;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

/* Note:
 * In multisession mode (when we synchronize more repositories in the same
 * session), we assume that the MailSyncListener is the main listener.
 * The main listener is in charge of performing operations related to the
 * beginning/end of the entire session (such as enabling/disabling sync button).
 */
public class MailSyncListener extends BasicSyncListener {

    /** Number of items received so far (in the current session) */
    private int received;
    /** Number of items sent so far (in the current session) */
    private int sent;
    /** Total number of items to be received in this session */
    private int toReceive;
    /** Total number of items to be sent in this session */
    private int toSend;
    private boolean alerted;
    /** Indicates if this sync is part of a multisession (sync of more
     * repositories)
     **/
    private boolean multisession;
    /** Session exit status */
    private int status;
    /** Indicates if this is the first block to be sent */
    private boolean firstSend;

    private int syncMode;
    
    private boolean isFirstMessageReceived;
    
    private ModalPopup popup;

    /** Creates a new instance of MailSyncListener */
    public MailSyncListener() {

        init();
    }

    public void itemReceived(Object item) {

        Log.debug(this, "Item received");
        
        // we need to reset the inbox but we don't want the user
        // to catch us performing in a slow sync required
        // from the server... so we refresh the  inbox 
        // only when we are doing a refresh from server (that
        // has probably been requested by our client)
        // reload of th inbox has been moved to itemReceived() method
        
        if ((syncMode == SyncML.ALERT_CODE_REFRESH_FROM_SERVER 
               || syncMode == SyncML.ALERT_CODE_SLOW)
               && isFirstMessageReceived) {
            Log.debug("[MailSyncListener] Reloading Message List");
            UIController.reloadInboxMessageListMessages();
            isFirstMessageReceived=false;
        }
        
                
        // Today folders that are received do not generate an event for the UI,
        // therefore we must be prepared to get here with a null item
        if (item == null) {
            return;
        }

        // In the future we may need to distinguish between a message and a
        // folder, but as of today we only need to notify messages
        Message msg = (Message) item;
        String userInfoMessage;

        // Update the number of received items
        received++;

        if (toReceive != ITEMS_NUMBER_UNKNOWN) {
            userInfoMessage = Localization.getMessages().
                    getMESSAGE_X_OF_Y_RECEIVED(received, toReceive);
        } else {
            userInfoMessage = Localization.getMessages().
                    getMESSAGE_X_RECEIVED(received);
        }
        UIController.updateInboxMessageList(userInfoMessage);

        Folder parent = msg.getParent();
        if (parent != null) {
            String parentName = parent.getName();
            // TODO: play sound only if the message has unread flag
            if ((Store.INBOX).equals(parentName)) {
                if (!alerted && (!msg.getFlags().isSet(MessageFlags.OPENED))) {
                    UserNotificationManager unm = UserNotificationManager.getInstance();
                    unm.playNewEmailTone();
                    unm.showNewEmailIcon();
                    alerted = true;
                }
                Folder folder = MessageManager.getInstance().getFolder(MessageManager.INBOX);
                InboxMessageList messageList =
                        UIController.getInboxMessageList();

                try {
                    messageList.addMessage(msg);
                } catch (MailException ex) {
                    Log.error("handleInboxMessage: " + ex.toString());
                    ex.printStackTrace();
                }
            }
        }
    }

    public void startSession() {

        Log.debug(this, "Start session");

        if (!multisession) {
            initializeSession();
        }
    }

    private void initializeSession() {
        //DISABLE GetMail Command during sync email
        UIController.enableSyncCommands(false);
        // Update msg
        String msg = Localization.getMessages().CONNECTING;
        UIController.updateInboxMessageList(msg);
    }

    public void endSession(int status) {

        Log.debug("[MailSyncListener] End session");
        this.status = status;

        // If this is not a multisession then we update the UI
        if (!multisession) {
            finalizeSession();
        }

        // At this stage of the synchronization the client has already
        // authenticated. It is therefore safe to start CTP (if not running
        // already). If authentication fails, what should we do?
        if (!CTPService.getInstance().isRunning()) {
            Log.debug("[MailSyncListener] CTP is not running, start it");
            UIController.midlet.startCTP();
        } else {
            Log.debug("[MailSyncListener] CTP is already running");
        }
    }

    public void endConnecting(int status) {

        Log.debug(this, "End connecting");

        // We anticipate the "Checking emails..." message as we want it to
        // last a little bit. If we display it in the startReceiving it gets
        // clobbered too soon by the "Receiving #"
        String msg = Localization.getMessages().LOADING;

        // Resetting inbox: the server answer OK to the 
        // Refresh from server request
        if (status == SyncML.ALERT_CODE_REFRESH_FROM_SERVER) {
            msg = Localization.getMessages().RESETTING_MSG;
            // Messages are deleted only after the connection has been 
            // established
            //UIController.messageManager.emptyFolder(UIController.messageManager.INBOX);
        }

        UIController.updateInboxMessageList(msg);
    }

    public void syncStarted(int syncMode) {
        this.syncMode=syncMode;
    }
    
    public boolean startSyncing(int syncMode) {
        if (syncMode==SyncML.ALERT_CODE_SLOW&&
                UIController.messageManager.getFolder(MessageManager.INBOX)
                                                         .getMessageCount()>0) {
            
            String title = Localization.getMessages().SLOW_SYNC_POPUP_TITLE;
            
            String msg = Localization.getMessages().SLOW_SYNC_POPUP_MSG;
            
            SlowSyncPopupAction sspa = new SlowSyncPopupAction();
            
                popup = new ModalPopup(title, msg, sspa);
            
            UIController.showModalPopup(popup, UIController.getInboxMessageList());
            
            synchronized(sspa) {
                try {
                    sspa.wait();
                } catch (InterruptedException ex) {
                    Log.error("Error waiting for user action" + ex); 
                }
            }
            
            UIController.showBackScreen();
            return sspa.isOkToSlowSync();
            
        }   
        
        return true;
    } 

    public void startReceiving(int numItems) {

        Log.debug(this, "Start receiving (" + numItems + ")");

        // Save the number of items to be received
        toReceive = numItems;
    }

    public void endReceiving() {

        Log.debug(this, "End receiving");
        toReceive = 0;
    }

    public void itemDeleted(Object item) {

        Log.debug(this, "Item deleted");
        String key = (String) item;
        //if message is in inbox, we delete it.
        UIController.getInboxMessageList().deleteMessage(key);
    }

    public void itemUpdated(Object item, Object update) {

        Log.debug(this, "Item updated by msgFlagsUpdated");

        // In some case the source does not perform the update and passes over
        // a null update object. In these cases we do not have to update
        // anything, so we ignore this event
        if (update == null) {
            return;
        }

        String key = (String) item;
        MessageFlags newFlags = (MessageFlags) update;

        //if message is in inbox, we update the flags
        UIController.getInboxMessageList().updateMessageFlags(key, newFlags);
    }

    public void itemUpdated(Object item) {

        String key = (String) item;
        String folder = MessageManager.getInstance().folderFromKey(key);

        // We only update items in the INBOX
        if (Store.INBOX.equals(folder)) {
            //MessageFlags newMask = FlagQueue.get(key);

            //if (newMask != null && (!newMask.isSet(MessageFlags.DELETED))) {
            Store store = StoreFactory.getStore();
            Folder inbox = store.getFolder(Store.INBOX);
            MessageManager mm = MessageManager.getInstance();
            Message msg = mm.getMessage(inbox, key);

            if (msg != null) {
                MessageFlags newFlags = msg.getFlags();

                UIController.getInboxMessageList().updateMessageFlags(key,
                        newFlags);
            } else {
                Log.info("[MailSyncListener] Message not found: " +
                        "It could have been deleted");
            }
        }

    /*if (Store.INBOX.equals(folder)) {
    MessageFlags newMask = FlagQueue.get(key);
    if (newMask != null && (!newMask.isSet(MessageFlags.DELETED))) {
    Store store = StoreFactory.getStore();
    Folder inbox = store.getFolder(Store.INBOX);
    MessageManager mm = MessageManager.getInstance();
    Message msg = mm.getMessage(inbox, key);
    MessageFlags newFlags = msg.getFlags();
    UIController.getInboxMessageList().updateMessageFlags(key,
    newFlags);
    }
    }*/
    }

    public void startSending(int numNewItems, int numUpdItems, int numDelItems) {

        Log.debug(this, "Start sending (" + numNewItems + "," + numUpdItems + "," + numDelItems + ")");
        toSend = numNewItems;

        //if we have something NEW to send. Other changes do not trigger the
        //SENDING message. In the future we may display a send updates msg
        if (toSend != 0 && firstSend) {
            // The number of items to be sent does not correspond to the number
            // of emails to be sent, but rather to the number of total changes.
            // Since this number can be misleading, at the moment we just
            // display a generic sending message
            String userInfoMessage = Localization.getMessages().SENDING_DOTTED;

            // TODO Is this really necessary? can we be outside a session???
            UIController.enableSyncCommands(false);
            UIController.updateInboxMessageList(userInfoMessage);
        }
    }

    public void itemAddSent(Object item) {

        // SyncItem syncItem = (SyncItem) item;
        Log.debug(this, "New item sent");
        String msg;
        if (toSend > 0) {
            // If the overall number of new items to send is known we show the
            // it in the message
            msg = Localization.getMessages().getSENDINGX_OF_Y(++sent, toSend);
        } else {
            msg = Localization.getMessages().getSENDINGX(++sent);
        }
        UIController.updateInboxMessageList(msg);
    }

    public void endSending() {

        Log.debug(this, "End sending");
        toSend = 0;
        firstSend = false;

    // TODO Why don't we re-enable the sync commands??? I think this must be
    // consistent with the above

    // We do not update the title here and wait for it to be rewritten at
    // the beginning of the receving phase. This way we avoid an annoying
    // transition Loading --> Inbox --> Loading and we keep the Loading.
    }

    public void dataReceived(String date, int size) {

        Log.debug(this, "Data received");

        // date like Thu, 03 May 2007 14:45:38 GMT
        try {
            if (!ConfigManager.getConfig().getTimeZoneUpdated()) {

                Date gmtServerdate = MailDateFormatter.parseRfc2822Date(date);
                Date deviceDate = new Date();

                int millisServerDate = (int) gmtServerdate.getTime();
                int millisDeviceDate = (int) deviceDate.getTime();
                int hourServer = (int) (millisServerDate / 3600000); //60*60*1000
                int hourDevice = (int) (millisDeviceDate / 3600000); //60*60*1000
                int hourDiff = hourDevice - hourServer;
                int minServer = (int) (millisServerDate / 60000); //60*60*1000
                int minDevice = (int) (millisDeviceDate / 60000); //60*60*1000
                int minDiff = minDevice - minServer - (hourDiff * 60);
                StringBuffer offset = new StringBuffer();
                offset.append((hourDiff > 0) ? "+" : "-");
                int mathHourDiff = (Math.abs(hourDiff));
                offset.append((mathHourDiff <= 9) ? "0" + mathHourDiff : "" + mathHourDiff);
                int mathMinDiff = (Math.abs(minDiff));
                if (mathMinDiff <= 20 || mathMinDiff >= 55) {
                    offset.append("00");
                } else if (mathMinDiff > 20 || mathMinDiff < 40) {
                    offset.append("30");
                } else if (mathMinDiff >= 40 || mathMinDiff < 55) {
                    offset.append("45");
                }

                // Save the new config.
                ConfigManager.getConfig().setTimeZone(offset.toString());
                ConfigManager.saveConfig();
            }
        } catch (ConfigException cex) {
            Log.error(this, "dataReceived - " + cex.toString());
        } catch (IOException ioex) {
            Log.error(this, "dataReceived - " + ioex.toString());
        }
    }

    /**
     * @return an error string based on the error status received
     */
    private static String getErrorMessage(int status) {
        Log.debug("Getting Error Message from error status: [" + status + "]");

        String message;
        switch (status) {
            case SyncSource.STATUS_CONNECTION_ERROR:
                message = Localization.getMessages().ACCOUNT_LISTENER_ERROR_CONNECTIONG;
                break;
            case SyncSource.STATUS_RECV_ERROR:
                message = Localization.getMessages().ACCOUNT_LISTENER_ERROR_RECEIVING;
                break;
            case SyncSource.STATUS_SEND_ERROR:
                message = Localization.getMessages().ACCOUNT_LISTENER_ERROR_SENDING;
                break;
            case SyncSource.STATUS_SERVER_ERROR:
                message = Localization.getMessages().ACCOUNT_LISTENER_ERROR_FROM_SERVER;
                break;
            default:
                message = Localization.getMessages().ACCOUNT_LISTENER_ERROR_DEFAULT;
                break;

        }
        return message;
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     * @param repositories repositories to be synchronized
     */
    public void startMultiSession(boolean repositories[]) {

        initializeSession();
        multisession = true;
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     */
    public void endMultiSession() {

        // At the end of the multisession we must reset the Inbox title with the
        // number of new messages received
        finalizeSession();
        multisession = false;
    }

    private void finalizeSession() {

        //String userInfoMessage;
        if (status != SyncSource.STATUS_SUCCESS && status == SyncSource.STATUS_SEND_ERROR) {
            //userInfoMessage = getErrorMessage(status);
            UIController.showErrorAlert(getErrorMessage(status), UIController.getInboxMessageList());
        }

        //else {
        // userInfoMessage = Localization.getMessages().getX_MESSAGES_RECEIVED(received);
        //UIController.updateInboxMessageList(Localization.getMessages().getX_MESSAGES_RECEIVED(received));
        //}
        // Update the title
        //UIController.updateInboxMessageList(userInfoMessage);
        UIController.updateInboxMessageList(Localization.getMessages().getX_MESSAGES_RECEIVED(received));

        UIController.enableSyncCommands(true);
    }

    private void init() {

        received = 0;
        sent = 0;
        toReceive = ITEMS_NUMBER_UNKNOWN;
        toSend = ITEMS_NUMBER_UNKNOWN;
        alerted = false;
        multisession = false;
        firstSend    = true;
        isFirstMessageReceived  = true; 
    }

   
    
    public class SlowSyncPopupAction implements PopupAction {
        private boolean okToSlowSync;
        
        private boolean actionPerformed = false;
        
        public SlowSyncPopupAction() {
            
        }
        
        public void cancel() {
            okToSlowSync=false;
            actionPerformed = true;
            synchronized(this) {
                this.notifyAll();
            }
        }
        
        public void confirm() {
            Log.info("User Confirmed slow sync action...");
            okToSlowSync=true;
            actionPerformed = true;
            synchronized(this) {
                this.notifyAll();
            }
        }
        
        public boolean isOkToSlowSync() {
            return okToSlowSync;
        }
        
        public boolean isActionPerformed() {
            return actionPerformed;
        }
    }
}
