/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.controller;

import javax.microedition.lcdui.*;

/**
 * This class can be used to implement a stack of displayable screens.
 * It provides a function to go forward and back with the screens
 * of the UI
 */
public class Stack {
    
    /** Array of screens in the Stack
     *  Configure the right dimension of the stack considering the depth of the
     *  UI workflow: i.e. how many back commands do we need to go back from the
     *  leaf to the root of the UI?
     */
    private Displayable [] screens;
    private final int STACK_DEPTH=20;
    
    public Stack() {
        screens = new Displayable [STACK_DEPTH];
    }
    
    /**
     * Insert a screen in the Stack
     * @param d Displayable screen to put in the Stack in order to retrieve later
     */
    public void push(Displayable d) {
        
        if ((d!=null) && (d!=screens[0]) && !(d instanceof Alert)) {
            for (int i=STACK_DEPTH-1; i>0;i--) {
                screens[i] = screens[i-1];
            }
            screens[0] = d;
        }
    }
    
    /**
     * Get the latest Screen to go back
     * @return Returns the Displayable Screen to display to go back
     */
    public Displayable pop() {
        Displayable result = screens[0];
        for (int i=0; i<STACK_DEPTH-2;i++) {
            screens[i] = screens[i+1];
        }
        screens[STACK_DEPTH-1] = null;
        return result;
    }
    
    public Displayable peek() {
        try {
            return screens[0];
        } catch (NullPointerException ex) {
            return null;
        }
    }

    public void clear() {
        for (int i=0; i<STACK_DEPTH-2;i++) {
            screens[i] = null;
        }    
    }
}
