/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

/*
 * Utils.java
 */

package com.funambol.mailclient.ui.utils;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.Folder;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

public  class UiUtils {
    
    private static String CONTINUE_STRING = "..";
    
    //private constructor to avoid instantiation
    private  UiUtils() {
    }
    
    /**
     * @return the best string to be drawn in the given region, i.e. the complete
     * string if it fits, or partial string with ".." if it does not fit
     *
     * @param text the text to be drawn
     * @g the graphics object
     * @w the width of the region
     *
     */
    //Should we put this somewhere else??
    public static String cutString(String text, int w, Font font) {
        if(font.stringWidth(text)<=w) {
            return text;
        } else {
            int width=0;
            for (int i=1;i<text.length();i++) {
                //width = font.substringWidth( text, 0, text.length()-i)+font.stringWidth(CONTINUE_STRING);
                
                width=font.stringWidth(text.substring(0,text.length()-i).trim())+font.stringWidth(CONTINUE_STRING);
                
                if (width<w) {
                    return text.substring(0,text.length()-i).trim()+ CONTINUE_STRING;
                }
            }
        }
        //we should never arrive here!
        return "";
    }
    
    /**
     * @return true if message is in inbox, false if in another folder or if no
     * folder is set as parent
     * @param message the message
     */
    public static boolean isIncoming(Message message) {
        Folder folder = MessageManager.getInstance().getFolder(MessageManager.INBOX);
        Folder parent = message.getParent();
        if (parent == null ) {
            return false;
        }
        return (folder.getName().equals(parent.getName()));
        
    }
    /**
     * @return true if message is in outbox or draft
     * @param message the message
     */
    public static boolean isOutgoing(Message message) {
        Folder outboxFolder = MessageManager.getInstance().getFolder(MessageManager.OUTBOX);
        Folder draftsFolder = MessageManager.getInstance().getFolder(MessageManager.DRAFTS);
        return (outboxFolder.getName().equals(message.getParent().getName()) ||
                draftsFolder.getName().equals(message.getParent().getName() ));
            
    }
    
    /**
     * @return true if message is in sent
     * @param message the message
     */
    public static boolean isSent(Message message) {
        Folder folder = MessageManager.getInstance().getFolder(MessageManager.SENT);
        return (folder.getName().equals(message.getParent().getName() )) ;
        
    }
    
    /**
     *
     * @param inDate the input date
     * @return the given date as a formatted string, if the date is in the
     * same day we are now it return the time, otherwise the day and month
     */
    public static String formatDate(Date inDate) {
        
        String date;
        Calendar now=Calendar.getInstance();
        Calendar sent=Calendar.getInstance();
        sent.setTime(inDate);
        if (now.get(Calendar.DAY_OF_MONTH)==sent.get(Calendar.DAY_OF_MONTH)
        && now.get(Calendar.MONTH)==sent.get(Calendar.MONTH) &&
                now.get(Calendar.YEAR)==sent.get(Calendar.YEAR) )
            date= MailDateFormatter.getFormattedStringFromDate(inDate,
                    MailDateFormatter.FORMAT_HOURS_MINUTES, ":");
        else
            date= MailDateFormatter.getFormattedStringFromDate(inDate,
                    MailDateFormatter.FORMAT_MONTH_DAY, "/");
        return  date;
    }
    
    /**
     * swaps two elements in the given array
     *
     * @param vector the vector containing the elements
     * @param element1 index of the first element
     * @param element2 index of the second element
     *
     */
    public static void swapElements(Vector vector, int element1, int element2) {
        
        Object tmp = vector.elementAt(element1);
        
        vector.setElementAt(vector.elementAt(element2), element1);
        vector.setElementAt(tmp, element2);
        
        
    }
    
    
     /**
     * get the addresses from the given message
     *
     * @return a vector of addresses containint the to, cc and bcc recipients
     * retrieved from the given message
     */
    public  static Vector getMessageAddresses(Message message) {
        if (message == null) {
            Log.info("[getMessageAddresses] message is null, returning empty vector");
            return new Vector(0);
        }
        
        Address[] toAddressList ;
        Address[] ccAddressList ;
        Address[] bccAddressList ;

        int toSize, ccSize, bccSize;
        
        try {
            toAddressList=message.getTo();
            toSize = toAddressList == null ? 0 : toAddressList.length;
            ccAddressList=message.getCc();
            ccSize = ccAddressList == null ? 0 : ccAddressList.length;
            bccAddressList=message.getBcc();
            bccSize = bccAddressList == null ? 0 : bccAddressList.length;
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("getMessageAddresses: "+ex.toString());
            return new Vector(0);
        }
        
        int size = toSize + ccSize + bccSize;
        Vector messageAddresses=new Vector(size);
        
        for (int i=0; i < toSize; i++) {
            messageAddresses.addElement(toAddressList[i]);
        }
        for (int i=0; i<ccSize; i++) {
            messageAddresses.addElement(ccAddressList[i]);
        }
        for (int i=0; i<bccSize; i++) {
            messageAddresses.addElement(bccAddressList[i]);
        }
        return messageAddresses;
    }
    
    /**
     * this method creates a vector of strings that can fit
     * on a single line of screenWidth pixel with given font.
     */
    public static String[] getStringArray(String text, int screenWidth, Font font) {
        // Log.debug(this, "GetStingArray() " + text.substring(0,10));
        
        Vector lines = new Vector();
        String [] ret;
        
        ChunkedString csLine = new ChunkedString(text);
        ChunkedString csWord;
        String[] lineSep = {"\r\n", "\n\r", "\r", "\n" };
        String[] wordSep = {" ", "\t"};
        
        
        //ChunkedString line;
        StringBuffer wordStringBuffer = new StringBuffer();
        String word;
        StringBuffer myLine=new StringBuffer();
        int widthLeft;
        while ( (csWord=csLine.getNextChunk(lineSep)) != null ) {
            //csWord = new ChunkedString(line);
            widthLeft = screenWidth;
            myLine.setLength(0);
            
            // get the words
            while ( (word = csWord.getNextString(wordSep)) != null ) {
                wordStringBuffer = new StringBuffer(word);
                
                if ( myLine.length() != 0 ) {
                    //if this is not the first word of the line we add a space
                    wordStringBuffer.insert(0," ");
                }
                int width = font.stringWidth(wordStringBuffer.toString());
                //if it fits inside the line
                if (width < widthLeft) {
                    //we append the word at the line and update widthleft
                    myLine.append(wordStringBuffer.toString());
                    widthLeft -=width;
                } else {
                    //if we've a space we can cut it out since this word
                    // will go to the next line
                    if (wordStringBuffer.charAt(0) == ' ') {
                        wordStringBuffer.deleteCharAt(0);
                    }
                    width = font.stringWidth(wordStringBuffer.toString());
                    //if can go on a new line
                    if ( width < screenWidth ) {
                        //we put it on a new line
                        lines.addElement(myLine.toString());
                        myLine.setLength(0);
                        myLine.append(wordStringBuffer.toString());
                        widthLeft = screenWidth - width;
                        continue;
                    } else {
                        int start = 0;
                        int end = 0;
                        int length = wordStringBuffer.length();
                        //String aString = myLine.toString();
                        while (start < length) {
                            // check length of the substring, adding a single char each time
                            for (int w = 0; w < widthLeft && end < length; end++) {
                                w = font.substringWidth( wordStringBuffer.toString(), start, (end - start) +1 );
                            }
                            myLine.append(wordStringBuffer.toString().substring(start,end));
                            if (end < length) {
                                lines.addElement(myLine.toString());
                                myLine.setLength(0);
                                widthLeft = screenWidth;
                            } else {
                                widthLeft = screenWidth - font.stringWidth(myLine.toString());
                            }
                            start = end;
                        }
                    }
                }
            }
            lines.addElement(myLine.toString());
        }
        ret = new String[lines.size()];
        //        Log.debug(this, "created vector, copying it");
        lines.copyInto(ret);
        lines=null;
        //       Log.debug(this, "returning string array");
        return ret;
    }
    
    
    private static Image resizeImage2(Image image, int width, int height) {
        int sourceWidth = image.getWidth();
        int sourceHeight = image.getHeight();
        
        Image thumb = Image.createImage(width, height);
        Graphics g = thumb.getGraphics();
        
        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                g.setClip(x, y, 1, 1);
                int dx = x * sourceWidth / width;
                int dy = y * sourceHeight / height;
                g.drawImage(image, x - dx, y - dy, Graphics.LEFT | Graphics.TOP);
            }
        }
        
        Image immutableThumb = Image.createImage(thumb);
        
        return immutableThumb;
    }
    
    
    private static int[] rescaleArray(int[] ini, int x, int y, int x2, int y2) {
        int out[] = new int[x2*y2];
        for (int yy = 0; yy < y2; yy++) {
            int dy = yy * y / y2;
            for (int xx = 0; xx < x2; xx++) {
                int dx = xx * x / x2;
                out[(x2*yy)+xx]=ini[(x*dy)+dx];
            }
        }
        return out;
    }
    
    
    public static Image resizeImage(Image image, int width, int height) {
        /*int width = getWidth();
        int height = width * image.getHeight() / image.getWidth();
         */
        
        //Need an array (for RGB, with the size of original image)
        int rgb[] = new int[image.getWidth()*image.getHeight()];
        //Get the RGB array of image into "rgb"
        image.getRGB(rgb,0,image.getWidth(),0,0,image.getWidth(),image.getHeight());
        //Call to our function and obtain RGB2
        int rgb2[] = rescaleArray(rgb,image.getWidth(),image.getHeight(),width,height);
        //Create an image with that RGB array
        Image temp2 = Image.createRGBImage(rgb2,width,height,true);
        
        return temp2;
    }
    
    public static Image bestFit(Image image, int maxWidth, int maxHeight) {
        
        // getting image properties
        int w = image.getWidth();
        int h = image.getHeight();
        
        //  get the ratio
        int ratiow = 100 * maxWidth / w;
        int ratioh = 100 * maxHeight / h;
        
        // this is to find the best ratio to
        // resize the image without deformations
        int ratio = Math.min(ratiow, ratioh);
        
        // computing final desired dimensions
        int desiredWidth = w * ratio/100;
        int desiredHeight = h * ratio/100;
        
        //resizing
        return  resizeImage(image, desiredWidth, desiredHeight);
    }
    
    
    
    /**
     * get the middle color given two colors
     * 
     * @param aStartColor the starting color
     * @param aEndColor the ending color
     * @return the middle color (sort of, no mapping in hsb is performed, so this
     *  is not really the middle color...)
     */
     public static int getMiddleColor(int aStartColor, int aEndColor) {

        int r1 = aStartColor>>16;
        int g1 = aStartColor>>8;
        g1 = g1 & 0x0000FF;
        int b1 = aStartColor;
        b1 = b1 & 0x0000FF;

        int r2 = aEndColor>>16;
        int g2 = aEndColor>>8;
        g2 = g2 & 0x0000FF;
        int b2 = aEndColor;
        b2 = b2 & 0x0000FF;

        int r = r1 - (r1 - r2) /2 ;
        r = r << 16;
        int g = g1 - (g1 - g2) /2 ;
        g = g << 8;
        int b = b1 - (b1 - b2) /2 ;

        return r | g | b;
    }
     
     public static int[] getGradient(int aStartColor, int aEndColor, int aLength){
        int r1 = aStartColor>>16;
        int g1 = aStartColor>>8;
        g1 = g1 & 0x0000FF;
        int b1 = aStartColor;
        b1 = b1 & 0x0000FF;
        
        int r2 = aEndColor>>16;
        int g2 = aEndColor>>8;
        g2 = g2 & 0x0000FF;
        int b2 = aEndColor;
        b2 = b2 & 0x0000FF;
        
        int res_r = 0;
        int res_g = 0;
        int res_b = 0;
        
        int gradient[] = new int[aLength];
        
        int counter = 0;
        int dif;
        while(counter < aLength){
            dif = (aLength-counter);
            res_r = (r1 * dif + r2*counter)/aLength ;
            res_g = (g1 * dif + g2*counter)/aLength ;
            res_b = (b1 * dif + b2*counter)/aLength ;
            
            res_r = res_r <<16;
            res_g = res_g <<8;
            
            gradient[counter] = (res_r | res_g | res_b);
            
            counter++;
        }
        return gradient;
    }
}
