/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


package com.funambol.mailclient.ui.view;

/**
 * rapresent a contact manager event
 */

public class ContactItemCollectionEvent {
    
    /**
     * the event code
     */
    private int eventCode;
    
    /**
     * the contactItem that the event refers to
     */
    private ContactItem contactItem;
    
    
    /**
     * code for noop
     */
    public static final int EVENT_CODE_NONE = 0;
    
    /**
     * contact has been added
     */
    public static final int EVENT_CODE_ADD_CONTACTITEM = 1;
    
    /**
     * contact has been updated
     */
    public static final int EVENT_CODE_UPDATE_CONTACTITEM = 2;
    
    /**
     * contact has been deleted
     */
    public static final int EVENT_CODE_DELETE_CONTACTITEM = 3;
    
    /**
     * code for contact list reset
     */
    public static final int EVENT_CODE_RESET_CONTACTITEM_LIST = 4;
    
    /**
     * contact list has been filtered
     */
    public static final int EVENT_CODE_FILTER_CONTACTITEM_LIST = 5;
    
    /**
     * contact list has been sorted
     */
    public static final int EVENT_CODE_SORT_CONTACTITEM_LIST = 6;

    /**
     * message has been changed
     */
    public static final int EVENT_CODE_MESSAGE_CHANGED = 7;
    
    //////////////////////////////////////////////////////////////////
    //    NOTE: if you need to add events constants, be sure to     //
    //    update the comments and the isValidEventCode method!      //
    // ///////////////////////////////////////////////////////////////
    
    
    /**
     *  Creates a new instance of ContactManageEvent with
     *  given  eventcode and contact
     *
     * @param eventCode the event code as one of
     * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACTITEM</li>
     *  <li>EVENT_CODE_UPDATE_CONTACTITEM</li>
     *  <li>EVENT_CODE_DELETE_CONTACTITEM</li>
     *  <li>EVENT_CODE_RESET_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_FILTER_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_SORT_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_MESSAGE_CHANGED</li>
     * </ul>
     */
    public ContactItemCollectionEvent(int eventCode, ContactItem contactItem) {
        this.eventCode = eventCode;
        this.contactItem = contactItem;
    }
    
    /**
     *  Creates a new instance of ContactManageEvent with
     *  evencCode EVENT_CODE_NONE and null contact.
     */
    public ContactItemCollectionEvent() {
        this(EVENT_CODE_NONE, null);
    }
    /**
     * @return the current contact, or null if no contact has been set
     */
    public ContactItem getContactItem() {
        return contactItem;
    }
    /**
     * set the current contactItem. null value is valid
     */
    public void setContactItem(ContactItem contactItem) {
        this.contactItem = contactItem;
    }
    /**
     *@return the current event code as one of
     * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACTITEM</li>
     *  <li>EVENT_CODE_UPDATE_CONTACTITEM</li>
     *  <li>EVENT_CODE_DELETE_CONTACTITEM</li>
     *  <li>EVENT_CODE_RESET_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_FILTER_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_SORT_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_MESSAGE_CHANGED</li>
     * </ul>
     */
    public int getEventCode() {
        return eventCode;
    }
    
    /**
     *  set the current event code.
     *  @param eventCode the event code to set. must be one of
     * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACTITEM</li>
     *  <li>EVENT_CODE_UPDATE_CONTACTITEM</li>
     *  <li>EVENT_CODE_DELETE_CONTACTITEM</li>
     *  <li>EVENT_CODE_RESET_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_FILTER_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_SORT_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_MESSAGE_CHANGED</li>
     * </ul>
     * if a different event code is used, this method does nothing
     */
    
    public void setEventCode(int eventCode) {
        if (isValidEventCode(eventCode)) {
            this.eventCode = eventCode;
        }
    }
    
    /**
     * @return true if the given eventcode is valid, i.e. is one of
      * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACTITEM</li>
     *  <li>EVENT_CODE_UPDATE_CONTACTITEM</li>
     *  <li>EVENT_CODE_DELETE_CONTACTITEM</li>
     *  <li>EVENT_CODE_RESET_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_FILTER_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_SORT_CONTACTITEM_LIST</li>
     *  <li>EVENT_CODE_MESSAGE_CHANGED</li>
     * </ul>
     */
    private boolean isValidEventCode(int eventCode) {
        return(
                eventCode == EVENT_CODE_ADD_CONTACTITEM ||
                eventCode == EVENT_CODE_DELETE_CONTACTITEM ||
                eventCode == EVENT_CODE_NONE ||
                eventCode == EVENT_CODE_RESET_CONTACTITEM_LIST ||
                eventCode == EVENT_CODE_UPDATE_CONTACTITEM || 
                eventCode == EVENT_CODE_FILTER_CONTACTITEM_LIST ||
                eventCode == EVENT_CODE_SORT_CONTACTITEM_LIST || 
                eventCode == EVENT_CODE_MESSAGE_CHANGED
                );
    }
    
}
