/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.cm.Contact;
import java.util.Vector;

/**
 * an interface to filter contactitems.
 * it consists a match method that returns true if the filter matches the 
 * given item and of a filter method that removes non matching items from the
 * given vector of items
 */
public interface ContactItemFilter {
    
    /**
     * @return true if the filter matches the given contactitem
     *
     * @param c the ContactItem to be filtered
     */
     
    public boolean match(ContactItem c);
    
    /**
     * return a vector with  matching elements.
     * @param fullContactItems the vector with the full list of contact items
     * @param partialContactItems the vector with a partial contact items list, 
     * used if the filter is incremental
     */
    public Vector filter(Vector fullContactItems, Vector partialContactItems)
    throws NoMatchingContactsException;
    
    /**
     * clear the current search.
     * after this method has been called the method match should return true for
     * every ContactItem
     */
    public void clear() ;
    
}
