/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;


import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;
import javax.microedition.rms.RecordStoreException;

/**
 * This list shows all the settings available to the user
 * for this application
 */
public class ContactPhoneNumbersList extends List implements CommandListener {
    
    private Command backCommand;
    private Command callCommand;
    private Contact contact;
    
    /** Creates a new instance of SettingsList */
    public ContactPhoneNumbersList(Contact contact) {
        super(contact.getNickName(), IMPLICIT);
         
        if(!StringUtil.isNullOrEmpty(contact.getMobilePhone())){
            this.append(contact.getMobilePhone(),Theme.getMOBILE_IMG());
        }
        if(!StringUtil.isNullOrEmpty(contact.getHomePhone())){
            this.append(contact.getHomePhone(),Theme.getHOME_IMG());
        }
        if(!StringUtil.isNullOrEmpty(contact.getJobPhone())){
            this.append(contact.getJobPhone(), Theme.getWORK_IMG());
        }
       
        backCommand = UIController.backCommand;
        addCommand(backCommand);
        callCommand = new Command("Call",UIController.COMMAND_TYPE,10);
        setSelectCommand(callCommand);
        addCommand(callCommand);
        setCommandListener(this);
    }
    
    public void commandAction(Command command, Displayable displayable) {
        if (command == callCommand) {
            try {
                UIController.midlet.platformRequest("tel:"+
                        this.getString(getSelectedIndex()));
            } catch (ConnectionNotFoundException ex) {
                ex.printStackTrace();
                Log.error("calling phonenumber: " + ex.toString());
                UIController.showErrorAlert("Error calling phonenumber.");
            } catch (Exception e) {
                e.printStackTrace();
                Log.error("calling phonenumber: " + e.toString());
            }
        } else if (command == backCommand) {
            UIController.showBackScreen();
        } 
    }
}

