/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 * Created on 11 gennaio 2007, 15.25
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.DeleteAllPopupAction;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.TextField;

public class DraftMessageList extends MessageList {

    private Command editCommand;
    private Command deleteAllCommand;
    private Command deleteCommand;

    /** Creates a new instance of DraftMessageList */
    public DraftMessageList(Vector msgInfo) {

        super(msgInfo);

    }

    
    
    protected void handleKey(int keyCode) {
        // handle the shortcut to delete a message
        if (UIController.isClearKey(keyCode)) {
            deleteMessage();
        }

        super.handleKey(keyCode);
    }

    /**
     * returns a default title
     */
    public String getDefaultTitle() {
        //if displayList.getNumItem()
        return Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE;
    }

    protected void addCommands() {
        super.addCommands();
        addCommand(UIController.backCommand);
        addCommand(UIController.inboxCommand);
    }

    /**
     * called by implementation when a list of elements with lenght >0 is set
     */
    protected void addItemCommands() {
        addCommand(editCommand);
        addCommand(deleteAllCommand);
        addCommand(deleteCommand);
        setFireCommand(editCommand);
    }

    /**
     * called by implementation when a list of elements with lenght = 0 is set
     */
    protected void removeItemCommands() {
        removeCommand(editCommand);
        removeCommand(deleteAllCommand);
        removeCommand(deleteCommand);
        setFireCommand(null);
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == UIController.inboxCommand) {
            UIController.showInboxScreen();
        } else if (command == UIController.backCommand) {
            // cannot use backScreen because if we open a draft msg it becomes the
            // new back screen and we cannot reach the folder list anymore
            //UIController.showBackScreen();
            UIController.showFolderList(UIController.getInboxMessageList(),
                    MessageManager.DRAFTS);
        } // ---- item commands --- //
        else if (command == editCommand) {
            setTitle(Localization.getMessages().OPENING_MESSAGE);
            this.repaint();
            this.serviceRepaints();
            UIController.updateComposeMessageScreen(getSelectedMessage(), "");
            UIController.showComposeMessageScreen(this, UIController.getComposeMessageScreen().body);
        } else if (command == deleteCommand) {
            deleteMessage();
        } else if (command == deleteAllCommand) {
            ModalPopup popup =
                    new ModalPopup(
                    Localization.getMessages().POPUP_DELETE_ALL_TITLE,
                    Localization.getMessages().POPUP_DELETE_ALL_TEXT,
                    new DeleteAllPopupAction(MessageManager.DRAFTS));
            UIController.showModalPopup(popup, this);
        } else {
            super.commandAction(command, displayable);
        }
    }

    protected void initCommands() {
        deleteAllCommand =
                new Command(Localization.getMessages().DML_DELETE_ALL_COMMAND, UIController.COMMAND_TYPE, 5);
        editCommand =
                new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE, 1);
        deleteCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 4);
    }
}
