/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Graphics;

/**
 *
 * simple drawing interface, implements some methods for drawing simple items
 * TODO: check if an (abstract) class may be better
 *
 */
public interface Drawer {
    
    /**
     * draw the canvas background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawCanvasBackground(Graphics g, int w, int h);
    
    
    
   /**
     * draw the title background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawTitleBackground(Graphics g, int w, int h);
    
    /**
     * draw a background in the given region
     *
     *@param g a graphics object
     *@param w the width of the region to be drawn
     *@param h the height of the region to be drawn
     *@selected is the item selected or not?
     */
    void drawBackground(Graphics g, int w, int h, boolean selected);

    /**
     * draw a background in the given region
     *
     *@param g a graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param w the width of the region to be drawn
     *@param h the height of the region to be drawn
     *@selected is the item selected or not?
     */
    void drawBackground(Graphics g, int x, int y, int w, int h, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     *@param graphics the graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     *@param graphics the graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param layout the layout described with combination of the Graphics layout constants
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, int layout, boolean selected);
    
    
    /**
     * draw a string using bold font in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawBoldString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * @returns the current graphicaltheme
     */
    GraphicalTheme getGraphicalTheme();
    
}
