/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.List;

import com.funambol.mail.Folder;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.MailException;

/**
 * a class that display the folders
 */
public class FolderList extends List implements CommandListener{
    
    private MessageManager mm;
    private Command selectCommand;
    
    /** Creates a new instance of FolderList
     * @param folders the array of folderinfo
     */
    public FolderList() {
        super(Localization.getMessages().FOLDER_LIST_TITLE, List.IMPLICIT);
        mm = UIController.messageManager;
        
        append(Localization.getMessages().INBOX_DEFAULT_TITLE, Theme.getINBOX_IMG());
        append(Localization.getMessages().OUTBOX_MESSAGE_LIST_TITLE, Theme.getOUTBOX_IMG());
        append(Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE, Theme.getDRAFT_IMG());
        append(Localization.getMessages().SENT_MESSAGES_LIST_TITLE, Theme.getSENT_IMG());
        
        
        selectCommand=new Command(Localization.getMessages().OPEN_COMMAND, UIController.COMMAND_TYPE, -1);
        addCommand(selectCommand);
        addCommand(UIController.backCommand);
        setSelectCommand(selectCommand);
        
        //#ifdef isBlackberry
//#        addCommand(UIController.blackberryExitCommand);
        //#endif
        setCommandListener(this);
    }
    
    /** setting updated message counters for each folder
     */
    public void updateCounters() {
        set(0, Localization.getMessages().INBOX_DEFAULT_TITLE + 
                getFolderCounter(mm.INBOX), Theme.getINBOX_IMG());
        set(1, Localization.getMessages().OUTBOX_MESSAGE_LIST_TITLE + 
                getFolderCounter(mm.OUTBOX), Theme.getOUTBOX_IMG());
        set(2, Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE +
                getFolderCounter(mm.DRAFTS), Theme.getDRAFT_IMG());
        set(3, Localization.getMessages().SENT_MESSAGES_LIST_TITLE +
                getFolderCounter(mm.SENT), Theme.getSENT_IMG());
    }
    /**
     * commandlistener
     */
    
    //TODO: should we use a UIController method (to decouple this from the
    //MessageManager ? )
    public void commandAction(Command command, Displayable displayable) {
        if (command==selectCommand){
            switch (getSelectedIndex()) {
                case MessageManager.INBOX:
                    UIController.showInboxScreen();
                    break;
                case MessageManager.OUTBOX:
                    UIController.showOutboxScreen(this);
                    break;
                case MessageManager.DRAFTS:
                    UIController.showDraftScreen(this);
                    break;
                case MessageManager.SENT:
                    UIController.showSentScreen(this);
            }
        } else if (command == UIController.backCommand) {
            UIController.showBackScreen();
        } else if (command == UIController.blackberryExitCommand) {
            UIController.midlet.destroyApp(false);    
        }
    }
    
    public void setSelectedFolder(int index) {
        this.setSelectedIndex(index, true);
    }

    private String getFolderCounter(int folder) {
        Folder f = mm.getFolder(folder);
        int counter = f.getMessageCount();
        int unread  = 0;

        if (folder == mm.INBOX) {
            try {
                unread = UIController.getInboxUnreadCounter();
            } catch (MailException me) {
                Log.debug("[FolderList] Unable to get message count from folder");
            }
        }

        
        
        GraphicalTheme theme = UIController.getDrawer().getGraphicalTheme();
       
         // bolding font if unread messages are present
        if (counter > 0) {
            setFont(folder, theme.getBoldFont());
                    
            if (unread > 0) {
                return " ("+unread+"/"+counter+")";
            } else {
                return " ("+counter+")";
            }
        } else {
            setFont(folder, theme.getDefaultFont());
            return "";
        }
    }
}
