/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

public class FunambolAlert extends Alert implements CommandListener {
    
    //default timeout in milliseconds
    public static final int DEFAULT_TIMEOUT = 3000;  //3 sec
    private Command detailsCommand;
    private Command okCommand;
    private Command dismissCommand;
    private boolean done;
    private Form detailsForm;
    private Displayable displayableToShow;
    public static boolean isShown;

    /** Creates a new instance of FunambolAlert */
    //public FunambolAlert(String title, String alertText, AlertType alertType) {
    public FunambolAlert(String title, String alertText, AlertType alertType) {
        super(title, alertText, Theme.getALERT_IMG(), alertType);
        if(displayableToShow == null){
                displayableToShow = UIController.display.getCurrent();
            }
        okCommand = 
                new Command(Localization.getMessages().GENERIC_OK, 
                    Command.BACK, 1);
        this.addCommand(okCommand);
        this.setCommandListener(this);
    }

    /** Creates a new instance of FunambolAlert with a long message to show 
     *  using details command
     */
    public FunambolAlert(String title, String smallMessage, String longMessage, AlertType alertType) {
        super(title, smallMessage, Theme.getALERT_IMG(), alertType);
        displayableToShow = UIController.display.getCurrent();
        detailsCommand = 
                new Command(Localization.getMessages().DETAILS_COMMAND_LABEL, 
                    Command.OK, 10);
        
        okCommand = 
                new Command(Localization.getMessages().GENERIC_OK, 
                    Command.BACK, 1);
        
        this.addCommand(detailsCommand);
        this.addCommand(okCommand);
        this.setCommandListener(this);
        detailsForm = new Form(Localization.getMessages().DETAILS_COMMAND_LABEL);
        detailsForm.append(longMessage);
        detailsForm.addCommand(UIController.backCommand);
        
        detailsForm.setCommandListener(this);
    }

    /**
     * Shows this alert as a modal screen. Not only the screen does not
     * disappear after some time, but the call is also blocking and the method
     * returns only after the user presses "OK".
     *
     * @param display the display
     */
    public void showModal(Display display) {
        setCommandListener(this);
        display.setCurrent(this);
        
        done = false;
        while (!done) {
            UIController.sleep(500);
        }
        isShown = false;
    }

    /**
     * Command listener for modal alerts.
     *
     * @param c the command
     * @param d the displayable
     */
    public void commandAction(Command c, Displayable d) {
        if (c == detailsCommand) {
            UIController.display.setCurrent(detailsForm);
        } else if (c == UIController.backCommand) {
            isShown = false;
            UIController.display.setCurrent(displayableToShow);
        } else if (c == okCommand) {
            isShown = false;
            UIController.display.setCurrent(displayableToShow);
        } 
    }
    
    public void setNextScreen(Displayable displayable){
        displayableToShow = displayable;
    }
}
