/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.Folder;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;

import com.funambol.mail.MessageFlags;
import com.funambol.mailclient.Funambol;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.Theme;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UserNotificationManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.AppProperties;
import com.funambol.push.CTPService;
//#ifdef isBlackberry
//# import com.funambol.util.BlackberryHelper;
//#endif
import com.funambol.util.Log;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.rms.RecordStoreException;

public class InboxMessageList extends MessageList implements Runnable {

    private Command openCommand;
    private Command composeCommand;
    private Command flagCommand;
    private Command unflagCommand;
    private Command foldersCommand;
    private Command settingsCommand;
    private Command helpCommand;
    private Command exitCommand;
    private Command syncCommand;
    private Command deleteCommand;
    private Command photoCommand;
    private boolean activatedFlagCommand;
    private Command backgroundCommand;
    private boolean isLoadingMessages = false;

    /** Creates a new instance of InboxMessageList */
    public InboxMessageList(Vector msgInfo) {
        super(msgInfo);
        try {
            Folder folder = MessageManager.getInstance().getFolder(MessageManager.INBOX);
            setMinimumDisplayedMessageCount(folder.getMessageCount());

        } catch (MailException ex) {
            Log.error(this, "unable to open inbox for message count");
            ex.printStackTrace();
        }
        if (UIController.isSyncPermitted()) {
            enableSyncCommand(true);
        }

    }

    public String getDefaultTitle() {

        StringBuffer title = new StringBuffer(Localization.getMessages().INBOX_DEFAULT_TITLE);
        // If there are unread messages, then we show their number
        int unread = UIController.getInboxUnreadCounter();
        if (unread > 0) {
            title.append(" (").append(unread).append(")");
        }
        return title.toString();
    }

    protected void addCommands() {
        super.addCommands();
        addCommand(composeCommand);
        addCommand(foldersCommand);
        addCommand(settingsCommand);
        addCommand(helpCommand);

        //#ifndef noPhoto
        // managing sendPhoto feature for releases at runtime
        String rel = UIController.appProperties.get(AppProperties.REL);
        if (rel != null && !rel.equals("")) {
            if (rel.equals("cared")) {
                addCommand(photoCommand);
            }
        }
        //#endif

//#ifndef JVMExitCommand
        //nokia 6630 adds an exit command by default, so we don't need to add our
        addCommand(exitCommand);
//#endif

//#ifdef backgroundCommand
//#     addCommand(backgroundCommand);
//#endif
    }

    // remove the commands e.g. when we need to go in ad mode
    protected void removeCommands() {
        removeCommand(composeCommand);
        removeCommand(foldersCommand);
        removeCommand(settingsCommand);
        removeCommand(helpCommand);

        //#ifndef JVMExitCommand
        //nokia 6630 adds an exit command by default, so we don't need to add our
        removeCommand(exitCommand);
    //#endif

    }

    /**
     * add the ads command
     */
    /**
     * called by implementation when a list of elements with lenght >0 is set
     */
    protected void addItemCommands() {
        addCommand(openCommand);
        addCommand(deleteCommand);
        setFireCommand(openCommand);
    }

    /**
     * called by implementation when a list of elements with lenght = 0 is set
     */
    protected void removeItemCommands() {
        removeCommand(flagCommand);
        removeCommand(unflagCommand);
        removeCommand(openCommand);
        removeCommand(deleteCommand);
        setFireCommand(null);
    }

    public void commandAction(Command command, Displayable displayable) {

        if (command == composeCommand) {
            UIController.composeNewMessage(UIController.display.getCurrent());
        } else if (command == foldersCommand) {
            UIController.showFolderList(this);
        } else if (command == settingsCommand) {
            UIController.showSettings(this);
        } else if (command == helpCommand) {
            UIController.showHelp(this);
        } else if (command == exitCommand) {
            UIController.midlet.destroyApp(false);
        } else if (command == syncCommand) {
            UIController.sync(UIController.isFirstSync);
        } // ---- item commands --- //
        else if (command == openCommand) {
            // Use method getFunTitle instead of getTitle
            // for double titlebar in Nokia S40
            String origTitle = getFunTitle();
            setTitle(Localization.getMessages().OPENING_MESSAGE);
            this.repaint();
            this.serviceRepaints();
            UIController.showViewMessageScreen(this, getSelectedMessage());
            setTitle(origTitle);
        } else if (command == flagCommand || command == unflagCommand) {
            if (getSelectedMessage() != null) {
                UIController.changeFollowUpFlag(getSelectedMessage());
                activate(getActiveElementId());
            }
        } else if (command == deleteCommand) {
            deleteMessage();
        } else if (command == photoCommand) {
            UIController.showTakePhotoScreen(this);
        } else if (command == backgroundCommand) {
            UIController.background();
        } //Moved to the end so this would fall through properly
        else {
            super.commandAction(command, displayable);
        }
    }

    /*private void deleteMessage() {
    Message msg = getSelectedMessage();
    if (msg!=null) {
    if (UIController.getConfig().getDeleteConfirmation()) {
    FunModalPopup popup =
    new FunModalPopup(Localization.getMessages().POPUP_DELETE_MESSAGE_TITLE,
    Localization.getMessages().POPUP_DELETE_MESSAGE_TEXT,
    new DeletePopupAction(msg));
    UIController.showModalPopup(popup, this);
    } else {
    UIController.deleteMessage(msg);
    }
    }
    }*/
    protected void initCommands() {
        int syncCmdType = Command.BACK;
        //#ifdef UseOKCmd
        //#         syncCmdType = UIController.COMMAND_TYPE;
        //#endif

        //#ifdef isBlackberry
//#        openCommand =
//#            new Command(Localization.getMessages().OPEN_COMMAND_LABEL,
//#            UIController.COMMAND_TYPE,-1);
        //#else
        openCommand =
                new Command(Localization.getMessages().OPEN_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 8);
        //#endif

        composeCommand =
                new Command(Localization.getMessages().COMPOSE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 10);
        foldersCommand =
                new Command(Localization.getMessages().FOLDERS_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 40);
        settingsCommand =
                new Command(Localization.getMessages().OPTIONS_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 50);
        helpCommand =
                new Command(Localization.getMessages().HELP_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 60);
        exitCommand =
                new Command(Localization.getMessages().EXIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 70);
        syncCommand = new Command(Localization.getMessages().SYNC_COMMAND_LABEL, syncCmdType, 5);

        deleteCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 20);
        flagCommand =
                new Command(Localization.getMessages().FLAG_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 30);
        unflagCommand =
                new Command(Localization.getMessages().UNFLAG_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 30);
        photoCommand =
                new Command(Localization.getMessages().PHOTO_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 20);

        //#ifdef isBlackberry
        //# backgroundCommand = new Command(Localization.getMessages().BB_BACKGROUND_COMMAND, Command.BACK, 99);
        //#else
        backgroundCommand = new Command(Localization.getMessages().STD_BACKGROUND_COMMAND, UIController.COMMAND_TYPE, 69);
    //#endif
    }

    public void run() {
        load();
    }

    public void load() {
        /* displayInboxMessageList(false); */

        // loading opened and closed image, they'll be probably needed later and
        // this minimize the empty inbox for devices like nokia 6630
        Theme.getImageFromFlags(MessageFlags.OPENED);
        Theme.getImageFromFlags(0);

        isLoadingMessages = true;
        int i = 0;
        MessageManager mm = MessageManager.getInstance();

        try {
            Folder f = mm.getFolder(MessageManager.INBOX);

            int tot = f.getMessageCount();
            String[] s = {Localization.getMessages().INBOX_LOADING, " "};

            Message msg = mm.getFirstMessage(f);
            while (msg != null) {

                addMessage(msg);
                /*if (i == 10) {
                UIController.display.setCurrent(this);
                }*/
                i++;
                s[1] = "(" + i + "/" + tot + ")";
                UIController.midlet.setSplashPhase(s, 0);
                msg = mm.getNextMessage(f);
            }

            UIController.midlet.setSplashPhase("Opening Inbox", 0);

        } catch (Exception e) {
            // can happen if a message is deleted while we're in the  cycle
            Log.error("Exception loading message " + i + " from inbox. " +
                    "Some messages have probably been deleted.");
            e.printStackTrace();
        } finally {
            setMinimumDisplayedMessageCount(COUNT_NONE);
            UIController.display.setCurrent(this);
            isLoadingMessages = false;
        }
    }

    /**
     * display the inbox message list reloading messages from RMS
     * @param resetTitle if true the title is set to the default title
     */
    public void displayInboxMessageList(boolean resetTitle) {
        setMessageList(UIController.getSortedMessages(MessageManager.INBOX));

        if (resetTitle) {
            resetTitle();
        }
        UIController.display.setCurrent(this);
    }

    protected void handleKey(int keyCode) {
        // handle the shortcut to flag a message with KEY_STAR key (*)
        Log.debug("keyCode= " + keyCode);
        Log.debug("getGameAction(keyCode)= " + getGameAction(keyCode));
        if (keyCode == KEY_STAR && !ad.isEnabled()) {
            if (activatedFlagCommand) {
                commandAction(flagCommand, this);
            } else {
                commandAction(unflagCommand, this);
            }


        } /*
        //TODO: TOREMOVE once debug is done
        //#ifdef isBlackberry
        //# 
        //#  else if (keyCode == KEY_NUM1) {
        //#     Log.debug("saving httpconfig 0 ");
        //#     BlackberryHelper.saveHttpConfig(0);
        //#  } 
        //#  else if (keyCode == KEY_NUM2) {
        //#     Log.debug("saving httpconfig 1 ");
        //#    BlackberryHelper.saveHttpConfig(1);
        //# } 
        //# else if (keyCode == KEY_NUM3) {
        //#    Log.debug("saving httpconfig 2 ");
        //#    BlackberryHelper.saveHttpConfig(2);
        //# } 
        //# else if (keyCode == KEY_NUM4) {
        //#    Log.debug("saving httpconfig 3 ");
        //#    BlackberryHelper.saveHttpConfig(3);
        //# } 
        //#endif
        /* else if (keyCode == KEY_NUM0) {
        Log.debug("forcing disconnect of ctp");
        CTPService.getInstance().forceDisconnect();
        } else if (keyCode == KEY_NUM8) {
        Log.debug("forcing restart of ctp");
        CTPService.getInstance().restartService();
        } else if (keyCode == KEY_NUM7) {
        Log.debug("forcing stop of ctp");
        CTPService.getInstance().stopService();
        } else if (keyCode == KEY_NUM9) {
        Log.debug("forcing start of ctp");
        CTPService.getInstance().startService();
        }*/ // end TOREMOVE once debug is done
        // handle the shortcut to delete a message
        else if (UIController.isClearKey(keyCode)) {
            deleteMessage();
        }

    }

    protected void keyPressed(int keyCode) {
        if (!UIController.isSyncInProgress()) {
            setTitle(getDefaultTitle());
        }

        super.keyPressed(keyCode);
    }

    protected void activate(int activeElement) {
        super.activate(activeElement);
        boolean activeMessageFlag = false;

        Message m = getSelectedMessage();
        if (m != null) {
            activeMessageFlag = m.getFlags().isSet(MessageFlags.FLAGGED);
            removeCommand(flagCommand);
            removeCommand(unflagCommand);
            if (activeMessageFlag) {
                activatedFlagCommand = false;
                addCommand(unflagCommand);
            } else {
                activatedFlagCommand = true;
                addCommand(flagCommand);
            }
        }
    }

    public void updateMessageFlags(String msgId, MessageFlags flags) {
        int p = search(msgId);
        if (p >= 0) {
            Message msg = getMessageAt(p);
            // Log.debug(this, "updating message flags for message " + msgId);
            msg.setFlags(flags);
            countUnread();
            repaint();
        } else {
            // Log.debug(this, "message " + msgId + " not found");
        }
    }

    /**
     * enable or disable the sync command and the reset inbox command
     * @param enable if true enable the command, if false disable the commands
     */
    public void enableSyncCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable) {
            addCommand(syncCommand);
        } else {
            removeCommand(syncCommand);
        }
    }

    /**
     * @return true if the inbox is loading messages (i.e. the inbox is in the 
     * run() method)
     */
    public boolean isLoadingMessages() {
        return isLoadingMessages;
    }

    /**
     * force the ad to move to the next ad
     */
    public void toNextAd() {
        ad.toNextAd();
    }

    protected void setElements(Vector messages) {
        super.setElements(messages);
        countUnread();
    }

    public void addMessage(Message msg) {

        super.addMessage(msg);
        countUnread();
    }

    public void deleteMessageAt(int position) {
        super.deleteMessageAt(position);
        countUnread();
    }

    public void paint(Graphics graphics) {
        // Since we are repainting the inbox, we consider messages as being read
        UserNotificationManager.getInstance().hideNewEmailIcon();
        super.paint(graphics);
    }

    private void countUnread() {
        // For performance reasons we do not use the Folder method to count
        // the number of messages with the OPENED flag unset. Because that
        // would require accessing the storage, while we have the list of
        // messages already available here.
        int unread = 0;

        for (int i = 0; i < elements.size(); ++i) {
            Message msg = (Message) elements.elementAt(i);
            MessageFlags flags = msg.getFlags();
            if (!flags.isSet(MessageFlags.OPENED)) {
                ++unread;
            }
        }
        UIController.setInboxUnreadCounter(unread);
    }
}
