/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
/**
 * MessageList.java
 */
package com.funambol.mailclient.ui.view;

import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import com.funambol.mail.Address;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.push.CTPService;

public class MessageList extends List implements CommandListener {

    /**
     * string to use when cutting a long string
     */
    private static final String CONTINUE_STRING = "..";
    /**
     * the margin
     */
    private static final int MARGIN = 3;
    /**
     * constant for setting no minimal displayed message count
     */
    public static final int COUNT_NONE = -1;
    /**
     * the Message list
     */
    private Vector messageList;
    private static Font font;
    private static Font boldFont;
    private static Font titleFont;
    private static Drawer drawer;
    private int minimumDisplayedMessageCount = -1;

    //private static GraphicalTheme;
    //-------------------------------------------------------------- Constructor
    /**
     * create a new MessageList
     */
    public MessageList(Vector elements) {
        super(elements);
        this.drawer = UIController.getDrawer();
        this.font = drawer.getGraphicalTheme().getDefaultFont();
        this.boldFont = drawer.getGraphicalTheme().getBoldFont();
        this.titleFont = drawer.getGraphicalTheme().getTitleFont();
        this.messageList = new Vector(ConfigManager.getMaxMessageNumber());
        resetTitle();
        initializeMessageItems();
        setCommandListener(this);

    }

    //------------------------------------------------------------------ Methods
    /**
     * set the message list, and repaint the message list
     */
    public void setMessageList(Vector msgList) {

        setElements(msgList);
        initializeMessageItems();
        repaint();
    }

    /**
     * @return the num of messages
     **/
    public int getNumMessages() {
        return elements.size();
    }

    /**
     * set the minimum displayed message count.
     * if set to COUNT_NONE the number of elements in the message
     * list is displayed
     *
     * @param i the minimum message count to be returned by
     * getDisplayedMessageCount. i must be >=0 or COUNT_NONE
     *
     */
    public void setMinimumDisplayedMessageCount(int i) {
        this.minimumDisplayedMessageCount = i < 0 ? COUNT_NONE : i;
    }

    /**
     * @return the selected message info
     */
    public Message getSelectedMessage() {
        if (getActiveElementId() < elements.size() && getActiveElementId() >= 0) {
            return (Message) elements.elementAt(getActiveElementId());
        } else {
            return null;
        }
    }

    /**
     *  paint the element
     */
    protected int paintElement(int elementId, Graphics g) {

        // this should not happen, but shomehow it happens on some
        // nokias (like 6111)
        if (elementId >= messageList.size()) {
            // skipping this
//            Log.info("Avoiding ArrayIndexOutOfBounds for element" + elementId);
            return 0;
        }


        g.translate(0, current_paint_h);

        int h = ((CanvasMessageItem) messageList.elementAt(elementId)).paint(g);

        g.translate(0, -current_paint_h);

        return h;
    }

    /**
     * create the messagelist from the array of elements
     */
    private void initializeMessageItems() {
        messageList.removeAllElements();
        //messageList.en
        for (int i = 0; i < elements.size(); i++) {
            messageList.addElement(new CanvasMessageItem(
                    (Message) elements.elementAt(i), i));
        }
    }

    /**
     * handle commandaction
     */
    public void commandAction(Command command, Displayable displayable) {

        if (command == UIController.backCommand) {
            UIController.showBackScreen();
        } else {
            super.commandAction(command, displayable);
        }
    }

    /**
     * paint the title and return the height of the painted area
     */
    public int paintTitle(Graphics g) {

        //   Log.debug("paint title!");

        String s;
        //String title = getTitle();

        // Use method getFunTitle instead of getTitle
        // for double titlebar in Nokia S40
        String title = getFunTitle();

        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        if (title == null) {
            //Log.debug("title null... setting to empty string");
            title = "";
        }
        s = " ";
        if (elements.size() > 0) {
            s += (getActiveElementId() + 1) + "/";
        }
        if (this.getDisplayedMessageCount() > 0) {
            s += this.getDisplayedMessageCount();
        } else {
            s = " " + this.getDisplayedMessageCount();
        }
        int sWidh = titleFont.stringWidth(s);

        String shortTitle = UiUtils.cutString(title, getWidth() - sWidh - 1, titleFont);
        drawer.drawTitleBackground(g, getWidth(), titleFont.getHeight());

        //-------- TODO remove this once basic testing is done
       /*
        
        if (CTPService.getInstance().isPushActive() ) {
        g.setColor(0x00dd00);
        } else {
        g.setColor(0xdd0000);    
        }
        g.fillRect(0, titleFont.getHeight()-3 , getWidth(), 3);
         */

        //-------- End of test code

        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        //not using drawTitle because we need to draw also the number of messages
        g.drawString(shortTitle, 1, 1, Graphics.TOP | Graphics.LEFT);
        g.drawString(s, getWidth(), 0, Graphics.TOP | Graphics.RIGHT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight();
    }

    /**
     * this method return the number of messages as shown in the title bar
     * if setminimumdisplayedmessagecount has been called, the returned
     * value is the max between minimumumDisplayedMessageCount (that can be
     * set using the setMinimumDisplayedMessageCount method) and the size of 
     * the message vector
     */
    public int getDisplayedMessageCount() {
        // ternary operator
        return minimumDisplayedMessageCount < elements.size() ? elements.size()
                : minimumDisplayedMessageCount;


    }

    /**
     * return the default title
     */
    public String getDefaultTitle() {
        return "";
    }

    /**
     * reset the title to the default title
     */
    public void resetTitle() {
        //    Log.debug(this, "resetting title to " + getDefaultTitle());
        setTitle(this.getDefaultTitle());
    }

    /**
     * Add a message to the current list of messages
     * @param message the message to be added
     */
    public void addMessage(Message message) {

        if (elements.size() == 0) {
            // This is the first message, we can use setMessageList
            Vector v = new Vector();
            v.addElement(message);
            setMessageList(v);
            //repaint();
            return;
        }

        int pos = search(message.getKey());

        if (pos >= 0) {
            // we overwrite previous messageinfo and canvasmessageitem's message
            ((CanvasMessageItem) messageList.elementAt(pos)).message = message;
            elements.setElementAt(message, pos);
            repaint();
            return;
        }


        // if we're here we need to add the message to the list in the
        // right position

        int i = 0;
        while (i < elements.size() && (message.getReceivedTime() <
                ((Message) elements.elementAt(i)).getReceivedTime())) {
            i++;
        }

        // we've found the right position.
        if (i == elements.size()) {
            //it's an append
            elements.addElement(message);
            messageList.addElement(new CanvasMessageItem(message, i));
        } else {
            //it's an insert
            elements.insertElementAt(message, i);
            messageList.insertElementAt(new CanvasMessageItem(message, i), i);

            // we change the id of each following item
            for (int j = i + 1; j < messageList.size(); j++) {
                //messageList.setElementAt(elements.elementAt(j-1),j);
                ((CanvasMessageItem) messageList.elementAt(j)).setId(j);
            }
        }

        // if we are showing too many elements, we remove them

        /*   while (elements.size() > ConfigManager.getMaxMessageNumber() ) {
        messageList.removeElementAt(elements.size()-1);
        elements.removeElementAt(elements.size()-1);
        }*/
        repaint();
    }

    /**
     * copy the objects in elements and messageList from the given position to
     * the given position
     * @param to the position to copy the objects to
     * @param from the position to copy the objects from
     */
    private void move(int to, int from) {

        elements.setElementAt(elements.elementAt(from), to);

        CanvasMessageItem mItem = (CanvasMessageItem) messageList.elementAt(from);
        mItem.setId(to);
        messageList.setElementAt(mItem, to);
    //messageList.elementAt()[to].setId(to);


    }

    /**
     * put the given messageinfo in the right position in
     * elements and messagelist
     *
     *@param to the position where to put the messageinfo
     *@param msg the message to be added
     */
    private void assign(int to, Message msg) {
        elements.setElementAt(msg, to);
        messageList.setElementAt(new CanvasMessageItem(msg, to), to);
    }

    /**
     * @return the index of the messageinfo with the same messagid as
     * the given messageinfo, or -1 if not found.
     */
    public int search(String msgId) {
        int position = -1;
        for (int i = 0; i < elements.size(); i++) {
            if (getMessageAt(i).getKey().equals(msgId)) {
                position = i;
                break;
            }
        }
        return position;

    }

    /**
     * handle the keypress events, should be overridden
     */
    protected void handleKey(int keyCode) {
    }

    /**
     *this method is called by implementation when the commands need to be
     * initialized (in the constructor), should be overridden. After this element
     * the commands referenced in addCommands, addItemCommands, removeItemcommands
     * must be not null
     */
    protected void initCommands() {
    }

    /**
     *this method is called by implementation when the generic commands need to be
     * added (in the constructor), should be overridden.
     */
    protected void addCommands() {
        super.addCommands();
    }

    /**
     *this method is called by implementation when the item commands need to be
     * added (i.e. while setting a non empty elements array.  should be ovverridden
     */
    void removeItemCommands() {
    }

    /**
     *this method is called by implementation when the item commands need to be
     * removed (i.e. while setting an empty elements array). should be ovverridden
     */
    void addItemCommands() {
    }

    /**
     * paint the background
     */
    void paintBackground(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }

    /**
     * delete the message eventually after a popup
     *
     */
    void deleteMessage() {
        Message msg = getSelectedMessage();
        if (msg != null) {

            if (UIController.getConfig().getDeleteConfirmation()) {
                ModalPopup popup =
                        new ModalPopup(Localization.getMessages().POPUP_DELETE_MESSAGE_TITLE,
                        Localization.getMessages().POPUP_DELETE_MESSAGE_TEXT,
                        new DeletePopupAction(msg));
                UIController.showModalPopup(popup, this);
            } else {
                UIController.deleteMessage(msg);
            }
        }
    }

    /**
     * remove the message in the given position from the visualized list.
     * This function will NOT delete the message from the folder
     *
     * @param position the position of the messageinfo to be removed
     */
    public void deleteMessageAt(int position) {

        //#ifdef low_mem
//#         UIController.freeMemory();      //XXX
        //#endif

        elements.removeElementAt(position);

        // Update messageList.
        messageList.removeElementAt(position);
        for (int i = position; i < messageList.size(); i++) {
            ((CanvasMessageItem) messageList.elementAt(i)).setId(i);
        }

        // If the deleted message was the last, set the new last as
        // active.
        if (getActiveElementId() >= elements.size()) {
            setActiveElement(elements.size() - 1);
        }

        repaint();
    }

    /**
     * Delete the message with the given msgID from the elements array, if present
     * this method does not delete the message from the rms!
     *
     * @param msgId the messageid of the message to be deleted
     */
    public void deleteMessage(String msgId) {
        int p = search(msgId);
        if (p >= 0) {
            deleteMessageAt(p);
        }
    }

    /**
     * Return the messageList
     */
    public Vector getMessageList() {
        return this.elements;
    }

    /**
     * Return the message at the given position
     * @param position the position of the element to be returned
     */
    public Message getMessageAt(int position) {
        return ((Message) elements.elementAt(position));
    }

//------------------------ CanvasMessageItem ---------------------------------//
    /**
     * this class draws the message item
     */
    class CanvasMessageItem {

        Message message;
        private int id;
        private int lastw;
        private String date;
        /**
         * the subject that fits on a single line
         */
        private String shortSubject;
        /**
         * the address that fits on a single line
         */
        private String shortAddress;
        /**
         * the height of this element when not expanded
         */
        private int unexpanded_h;
        /**
         * the height of this element when expanded
         */
        private int expanded_h;
        /**
         * track if the last time we painted the item the read flag was
         * set or not (to recalculate the short strings if necessary)
         */
        private boolean lastReadFlag;

        CanvasMessageItem(Message msg, int id) {
            this.message = msg;
            this.id = id;


        }

        private boolean isActive() {
            return id == getActiveElementId();
        }

        /**
         * paint the messageitem
         */
        public int paint(Graphics graphics) {

            //we need to calculate the short strings
            int w = getWidth();
            int h = calculateHeight();
            calculateShortStrings();

            //we draw the background, image and date
            drawer.drawBackground(graphics, w, h, isActive());
            graphics.drawImage(Theme.getImageFromFlags(getFlagValue()), MARGIN, MARGIN, Graphics.TOP | Graphics.LEFT);
            drawer.drawString(getDate(), graphics, w - MARGIN, MARGIN, Graphics.TOP | Graphics.RIGHT, isActive());

            //if this is a read message we use normal font, otherwhise we use the bold font
            if (isRead()) {

                drawer.drawString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH() +
                        2 * MARGIN, MARGIN, isActive());
            } else {
                drawer.drawBoldString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH() +
                        2 * MARGIN, MARGIN, isActive());
            }
            if (hasAttach()) {
                int attachPos = 0;
                if (isRead()) {
                    attachPos = Theme.getIMAGE_PART_WIDTH() +
                            3 * MARGIN +
                            font.stringWidth(shortAddress);
                } else {
                    attachPos = Theme.getIMAGE_PART_WIDTH() +
                            3 * MARGIN +
                            boldFont.stringWidth(shortAddress);
                }
                graphics.drawImage(Theme.getCLIP_IMG(), attachPos, MARGIN, Graphics.TOP | Graphics.LEFT);

            }
            //if expanded we draw the second line
            if (isExpanded()) {
                drawer.drawString(shortSubject, graphics, MARGIN, getUnexpandedh(), isActive());
            }
            return h;
        }

        /**
         * return the flag value used to identify the image in the image array
         */
        private int getFlagValue() {
            int flags = message.getFlags().getFlags();
            //we mask the flag to be sure we have a return value <32
            flags = (flags & 0x0000001f);
            return flags;
        }

        /**
         * calculate the short version of address and subject and
         * store them in shortAddress and shortSubject
         */
        private void calculateShortStrings() {

            if (shortAddress == null || shortSubject == null || lastReadFlag != isRead()) {

                int w = getWidth();
                int firstlineSpace = w - 4 * MARGIN - Theme.getIMAGE_PART_WIDTH() - font.stringWidth(getDate());
                if (hasAttach()) {
                    firstlineSpace = firstlineSpace - Theme.getCLIP_IMG().getWidth();
                }
                int secondlineSpace = w - MARGIN;
                shortSubject = UiUtils.cutString(message.getSubject(), secondlineSpace, font);

                String address = null;
                if (isIncomingMessage()) {
                    Address from = message.getFrom();
                    if (from != null) {
                        address = from.getVisibleName();
                    }
                } else {
                    Address to[] = message.getTo();
                    Address cc[] = message.getCc();
                    if (to != null && to.length > 0) {
                        Address singleTo = to[0];
                        address = singleTo.getVisibleName();
                    } else if (cc != null && cc.length > 0) {
                        Address singleCc = cc[0];
                        address = singleCc.getVisibleName();
                    }
                }
                if (address == null) {
                    address = Localization.getMessages().EMPTY_RECIPIENTS;
                }
                if (isRead()) {
                    shortAddress = UiUtils.cutString(address, firstlineSpace, font);
                    lastReadFlag = true;
                } else {
                    shortAddress = UiUtils.cutString(address, firstlineSpace, boldFont);
                    lastReadFlag = false;
                }
            //   Log.error("shortstring: " + shortSubject);

            } else {
                //   Log.error("no need to calculate shortstring");
            }


        }

        /**
         * @return true if message is in the inbox
         */
        private boolean isIncomingMessage() {
            return (Store.INBOX.equals(message.getParent().getName()));
        }

        /**
         * @return true if the message has been opened
         */
        private boolean isRead() {
            return message.getFlags().isSet(MessageFlags.OPENED);
        }

        /**
         * @return true if the message has one ore more attachments
         */
        private boolean hasAttach() {
            return message.isMultipart();

        }

        /**
         * return true if this item is expanded, false otherwise
         */
        public boolean isExpanded() {
            boolean ret;
            //#ifdef MessageItemAlwaysExpanded
//#             ret = true;
            //#else
            ret = isActive();
            //#endif
            return ret;
        }

        /**
         * compute height using information from the bean,
         * the drawer and the icon size
         */
        private int calculateHeight() {
            if (isExpanded()) {
                return getExpandedh();
            } else {
                return getUnexpandedh();
            }
        }

        /**
         * @retunr the height of this element not expanded
         */
        private int getUnexpandedh() {
            if (unexpanded_h == 0) {
                unexpanded_h = MARGIN;
                unexpanded_h += Math.max(
                        Math.max(font.getHeight(), boldFont.getHeight()), Theme.getIMAGE_PART_HEIGHT());
            //unselected_h+=MARGIN;
            }
            return unexpanded_h;
        }

        /**
         * @retunr the height of this element expanded
         */
        private int getExpandedh() {
            if (expanded_h == 0) {
                expanded_h = getUnexpandedh();
                expanded_h += font.getHeight();
                expanded_h += MARGIN;
            }
            return expanded_h;
        }

        //  new Vector()
        /**
         * return the date as a formatted string
         */
        private String getDate() {
            if (date == null) {
                date = UiUtils.formatDate(message.getReceivedDate());
            }
            return date;


        }

        /**
         *
         */
        private void setId(int newId) {
            this.id = newId;
        }
    }

    boolean isVisible(int elementId) {
        return elements.elementAt(elementId) != null;
    //return true;
    }

    boolean isActivable(int elementId) {
        return isVisible(elementId);
    }
}
