/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.AlreadyExistentContactException;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactListFullException;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemCommandListener;
import javax.microedition.lcdui.TextField;

/**
 * shows a list of the recipients of a given message.
 * each recipient can be saved as a contact
 **/
public class RecipientList extends ContactList {
    
    
    private Command saveContactCommand;
    
    /**
     * create a new instance of Recipientlist with the given message
     */
    public RecipientList(Message message) {
        super(message);
    }
    
    
    
    public void addCommands() {

        addCommand(saveContactCommand);
        addCommand(UIController.cancelCommand);

        //#ifdef isBlackberry
        //# addCommand(UIController.blackberryExitCommand);
        //#endif
    }
    
    /**
     * nothing to do here
     */
    protected void removeItemCommands() {
        ;
    }
    
    /**
     * nothing to be done here
     */
    protected void addItemCommands() {
    }
    
    protected void initCommands() {
        super.initCommands();

        // Disable these commands
        clearContactCommand = null;
        bccContactCommand   = null;
        ccContactCommand    = null;
        toContactCommand    = null;

        saveContactCommand =
                new Command(Localization.getMessages().SAVE_COMMAND, UIController.COMMAND_TYPE, 3);
    }


    /**
     * add the commands the searchbox, and the contacts
     */
    public void initializeContactList() {
    
        try {
            
            Vector list = UiUtils.getMessageAddresses(message);
            Vector contacts = new Vector(list.size());
            Contact c ;
            for (int i = 0; i<list.size(); i++) {
                c=new Contact((Address)list.elementAt(i));
                contacts.addElement(new ContactItem(
                        c,((Address)list.elementAt(i)).getType(), Contact.DEFAULT_EMAIL));
            }
            setContactItemList(contacts);
        } catch (ContactManagerException ex) {
            Log.error("error creating contact from one of the recipients");
            ex.printStackTrace();
        }
    }
    
    public void commandAction(Command command, Displayable d) {
        if (command == saveContactCommand) {
            // we need to reset the contactlist message to clean the
            // recipients
            UIController.getContactList().setMessage(new Message());
            UIController.showAddContactScreen(
                    UIController.display.getCurrent(),getActiveContact());
        }else  if (command==UIController.cancelCommand) {
            UIController.showBackScreen();
        } else {
            super.commandAction( command, d );
        }
    }
    
    protected void enableDisableOKCommand() {
        // nothing to do here!
        ;
    }

    
    void handleKey(int keyCode) {
      // doin' nothing
    }
    
    
    // empty method, returns 0 and does nothing
    protected int drawSubtitle(final Graphics g) {
    // no need to paint stuff here
    return 0;
    }
    
   

    public String getDefaultTitle() {
        
        return Localization.getMessages().RECIPIENT_LIST_TITLE;
    }
    
}
