/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;

import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.DeleteAllPopupAction;
import com.funambol.mailclient.ui.controller.UIController;


public class SentMessageList extends MessageList {
    
    private Command openCommand;
    private Command deleteAllCommand;
    private Command deleteCommand;
    

    
    /** Creates a new instance of SentMessageList */
    
    
    public SentMessageList(Vector msgList) {
        super(msgList);
    }
    
    public String getDefaultTitle() {
         return Localization.getMessages().SENT_MESSAGES_LIST_TITLE;
    }
    
    protected void addCommands() {
        super.addCommands();
        
        
        addCommand(UIController.backCommand);
        addCommand(UIController.inboxCommand);
        
    }
    
    /**
     * called by implementation when a list of elements with lenght >0 is set
     */
    protected void addItemCommands() {
        addCommand(openCommand);
        addCommand(deleteAllCommand);
        addCommand( deleteCommand );
        setFireCommand(openCommand);
    }
    
    /**
     * called by implementation when a list of elements with lenght = 0 is set
     */
    protected void removeItemCommands() {
        removeCommand(openCommand);
        removeCommand(deleteAllCommand);
        removeCommand(deleteCommand);
        setFireCommand(null);
    }

    protected void handleKey( int keyCode ) {

        /* Removed because handled in isClearKey method
         //ifdef isBlackberry 
        //keycode == 8 is delete key on blackberry
        // if (keyCode == 8 && getSelectedMessage() != null){
        //    commandAction(deleteCommand, this);
        //    return;
        // } 
        //endif*/
        
        // handle the shortcut to delete a message
        if (UIController.isClearKey(keyCode)) {
            deleteMessage();
        } 
        super.handleKey( keyCode );
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == UIController.inboxCommand) {
            UIController.showInboxScreen();
        } else if (command==UIController.backCommand) {

            UIController.showFolderList(UIController.getInboxMessageList(),
                    MessageManager.SENT);
        } 
        // ---- item commands --- //
        else if (command==deleteAllCommand) {
            ModalPopup popup =
                    new ModalPopup(Localization.getMessages().POPUP_DELETE_ALL_TITLE,
                    Localization.getMessages().POPUP_DELETE_ALL_TEXT,
                    new DeleteAllPopupAction(MessageManager.SENT));
            UIController.showModalPopup(popup, this);
        } else  if (command==openCommand) {
            setTitle(Localization.getMessages().OPENING_MESSAGE);
            this.repaint();
            this.serviceRepaints();
            UIController.showViewMessageScreen(this, getSelectedMessage());
            resetTitle();
        } else if (command == deleteCommand) {
            deleteMessage();
        }
        else {
            super.commandAction( command, displayable );
        }
    }
    
    protected void initCommands() {
        deleteAllCommand =
                new Command(Localization.getMessages().DML_DELETE_ALL_COMMAND,UIController.COMMAND_TYPE,4);
        openCommand =
                new Command(Localization.getMessages().IML_OPEN_COMMAND,UIController.COMMAND_TYPE,1);
        deleteCommand = new Command(Localization.getMessages().DELETE_COMMAND_LABEL,Command.OK,2);
    }
}
