/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

/*
 * ViewMessage.java
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.RMSStore;
import com.funambol.mail.Store;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.cm.rms.RMSContactManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import com.funambol.util.StringUtil;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class ViewMessage extends Canvas implements CommandListener {

    /**
     * the message
     */
    private Message message;
    private Drawer drawer;
    private GraphicalTheme theme;
    private Font bodyFont;
    private static final int SCROLL_AMOUNT = 2;
    private static final int MARGIN = 2;
    private static final int SCROLLBAR_WIDTH = 4;
    /**
     * are we inside the header?
     */
    private boolean insideHeader = true;
    /**
     * are we inside the body?
     */
    private boolean insideBody = false;
    private BodyItem bodyItem;
    private boolean showHeader = true;
    private HeaderItem headerItem;
    private Command deleteCommand;
    private Command forwardCommand;
    private Command linksCommand;
    //private Command numbersCommand;
    private Command fullMessageCommand;
    private Command recipientsCommand;
    private Command replyCommand;
    private Command replyAllCommand;
    private Command saveSenderCommand;
    private Command markAsReadCommand;
    private Command followUpCommand;
    private Command editCommand;
    private Command sendCommand;
    private Command callCommand;
    private Command attachList;
    private Contact contactToCall;
    private int NEXT_MESSAGE = 1;
    private int PREVIOUS_MESSAGE = -1;
    private String theTitle = Localization.getMessages().MESSAGE_VIEW_FORM_TITLE;
    private boolean fullMsgPerformed;

    private Ads ad;
    
    /**
     * creates a new FunCanvasViewMessage with given message
     */
    public ViewMessage(Message message) {
        ad = UIController.getAd();
        this.message = message;
        this.drawer = UIController.getDrawer();
        this.theme = drawer.getGraphicalTheme();
        addCommands();
        setCommandListener(this);
        bodyFont = theme.getDefaultFont();
        this.bodyItem = new BodyItem();
        this.headerItem = new HeaderItem();

        repaint();
    }

    /**
     * paint the viewmessagteitem
     */
    protected void paint(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
        //drawer.drawString("ciao", graphics, 0,0,false);
        int h= ad.paintSmallAd(graphics);
        graphics.translate(0,h);
        int titleheight = paintTitle(graphics);
        graphics.translate(0,-h);
        h+=titleheight;
        h = headerItem.draw(graphics, getWidth(), h);
        bodyItem.draw(graphics, getWidth() - SCROLLBAR_WIDTH - MARGIN, h);
        drawScrollBar(graphics, h);
    }

    /**
     * enable or disable the fullMessageCommand command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableFullMessageCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable && !fullMsgPerformed) {
            addCommand(fullMessageCommand);
        } else {
            removeCommand(fullMessageCommand);
        }
    }

    public int paintTitle(Graphics g) {

        //   Log.debug("paint title!");

        /*
         *
         *    int titleheight = drawer.getGraphicalTheme().getTitleFont().getHeight();
        drawer.drawTitleBackground(graphics, getWidth(), titleheight);
        drawer.drawString(graphics,1,1, false);
         */


        String s;

        Font titleFont = drawer.getGraphicalTheme().getTitleFont();
        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        if (theTitle == null) {
            //Log.debug("title null... setting to empty string");
            theTitle = "";
        }

        String shortTitle = UiUtils.cutString(theTitle, getWidth() - 1, titleFont);
        drawer.drawTitleBackground(g, getWidth(), titleFont.getHeight());
        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        g.drawString(shortTitle, 1, 1, Graphics.TOP | Graphics.LEFT);
        g.drawString(theTitle, 1, 1, Graphics.TOP | Graphics.LEFT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight() + 1;
    }

    public void setTitle(String title) {

        //Log.error("setting title to " + title);

        //we have to ignore setTitle (null) because some motorola devices
        //call this method when building the canvas
        //TODO: use ifdef
        if (title != null) {
            //      Log.debug("calls to setTitle (null): ignoring it");
            // Log.error("setting title to " + title);
            theTitle = title;
        }

    //don't do a repaint here, or Nokia 6630 will crash.

    }

    /*    public String getTitle() {
    return this.theTitle;
    }
     */
    /**
     *
     * @return the current title
     */
    // Method getTitle() renamed in getFunTitle()
    // for double titlebar bug in Nokia S40
    public String getFunTitle() {
        //    Log.debug(this, "getTitle called, returning the fake title '" + theTitle + "'");
        return theTitle;
    }

    private void exitBody(int dir) {
        //  Log.debug("exit body");
        if (dir == Canvas.UP) {
            insideBody = false;
            enterHeader(dir);
        }
    }

    private void enterHeader(int dir) {
        // Log.debug("enter header");
        insideHeader = true;
        showHeader = true;
    }

    private void exitHeader(int dir) {
        //  Log.debug("exit header");
        if ((dir == Canvas.DOWN)) {
            insideHeader = false;
            showHeader = false;
            enterBody(dir);
        }

    }

    /**
     * Handle the key navigation
     * @param keyCode is the key that has been pressed
     */
    protected void keyPressed(int keyCode) {
    
        if (keyCode == Canvas.KEY_STAR) {
           if (ad.isEnabled()) {
               UIController.handleAdClick(ad);
           } 
           
        }
        
        
        // shortcut for deleting message
        if (keyCode == Canvas.KEY_NUM0) {
            deleteMessage();
        } else if (keyCode == -10) {
            // green key
            if (isValidForCall(contactToCall)) {
                UIController.phoneCall(contactToCall, this);
            }
        }

        int dir = getGameAction(keyCode);
        if (dir == Canvas.DOWN || dir == Canvas.UP) {

            if (insideBody) {
                //   Log.debug("sending traverse to body Item");
                bodyItem.traverse(dir);
            } else if (insideHeader) {
                // Log.debug("sending traverse to header Item");
                headerItem.traverse(dir);
            }
            repaint();
        } else if (dir == Canvas.RIGHT) {
            try {
                this.setTitle(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE);
                repaint();
                serviceRepaints();
                getNextMessage(NEXT_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        } else if (dir == Canvas.LEFT) {
            try {
                this.setTitle(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE);
                repaint();
                serviceRepaints();
                getNextMessage(PREVIOUS_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        }
    }

    /**
     * Sets the correct message when left/rigth navigation is performed
     * @param shift is the
     */
    private void getNextMessage(int shift) {
        ad.toNextAd();
        MessageList fcml = UIController.getMessageList(message);
        int msgNumber = fcml.getNumMessages();
        int messageToSet = fcml.getActiveElementId() + shift;
        if (messageToSet >= msgNumber) {
            messageToSet = 0;
            fcml.rewindList();
            shift = 0;
        } else if (messageToSet < 0) {
            messageToSet = msgNumber - 1;
            shift = fcml.getNumMessages();
        }

        if (Math.abs(shift) > 1) {
            for (int i = 0; i < shift; i++) {
                fcml.scroll(1);
                fcml.repaint();
            }
        } else {
            fcml.scroll(shift);
        }
        fcml.setActiveElement(messageToSet);
        cleanup();
        UIController.showViewMessageScreen(
                fcml, fcml.getSelectedMessage());
    //repaint();
    }

    private void addCommands() {
        deleteCommand = new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 27);
        forwardCommand = new Command(Localization.getMessages().MVF_FORWARD_COMMAND, UIController.COMMAND_TYPE, 25);
        linksCommand = new Command(Localization.getMessages().MVF_LINKS_COMMAND, UIController.COMMAND_TYPE, 55);
        //numbersCommand=new Command(Localization.getMessages().MVF_NUMBERS_COMMAND, UIController.COMMAND_TYPE,65);

        fullMessageCommand = new Command(Localization.getMessages().MVF_FULL_MSG_CMD, UIController.COMMAND_TYPE, 2);
        callCommand = new Command(Localization.getMessages().CALL_SENDER_COMMAND, UIController.COMMAND_TYPE, 26);

        attachList = new Command(Localization.getMessages().ATTACHMENT_LIST_CMD, UIController.COMMAND_TYPE, 1);

        if (message.isMultipart()) {
            addCommand(attachList);
        }

        //#ifdef isBlackberry
        //# this.addCommand( UIController.nextCommand );
        //# this.addCommand( UIController.previousCommand );
        //# this.addCommand( UIController.blackberryExitCommand );
        //#endif

        //this commands are always present
        this.addCommand(UIController.backCommand);
        if (!UiUtils.isOutgoing(message)) {
            this.addCommand(forwardCommand);
            this.addCommand(linksCommand);
        }
        //this.addCommand(numbersCommand);

        // allows to send log message if the SyncClient is not running
        // This is present if the message is incomplete.
        if (message.getFlags().isSet(MessageFlags.PARTIAL)) {
            if (UIController.isSyncPermitted()) {
                this.addCommand(fullMessageCommand);
            }
        }


        //this are present if the message is incoming
        if (UiUtils.isIncoming(message)) {
            replyCommand = new Command(Localization.getMessages().MVF_REPLY_COMMAND, UIController.COMMAND_TYPE, 9);
            replyAllCommand = new Command(Localization.getMessages().MVF_REPLY_ALL_COMMAND, UIController.COMMAND_TYPE, 10);
            //flagCommand=new Command("Flag",Command.SCREEN,4);
            saveSenderCommand = new Command(Localization.getMessages().MVF_SAVE_SENDER_COMMAND, UIController.COMMAND_TYPE, 45);
            recipientsCommand = new Command(Localization.getMessages().MVF_RECIPIENTS_COMMAND, UIController.COMMAND_TYPE, 75);

            setMarkAsReadCommand();
            setFollowUpCommand();

            this.addCommand(deleteCommand);
            this.addCommand(replyCommand);
            int numRec = message.getNumberOfRecipients();

            // TODO: verify why the code below is needed
            Address[] to = message.getTo();
            for (int i = 0; i < to.length; i++) {
                if (!StringUtil.equalsIgnoreCase(to[i].getEmail(), UIController.getMyAddress())) {
                    numRec++;
                }
            }

            if (numRec > 1) {
                this.addCommand(replyAllCommand);

            }
            this.addCommand(recipientsCommand);
            //this.addCommand(flagCommand);
            this.addCommand(saveSenderCommand);
        } else if (UiUtils.isOutgoing(message)) {
            //outbox or draft
            this.addCommand(deleteCommand);
            //sendCommand = new Command(Localization.getMessages().SEND_COMMAND_LABEL, UIController.COMMAND_TYPE, 0);
            //editCommand = new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE, 1);
            //this.addCommand(sendCommand);
            //this.addCommand(editCommand);
        } else if (UiUtils.isSent(message)) {
            //sendCommand=new Command(Localization.getMessages("MVF_ResendCommand"),UIController.COMMAND_TYPE, 0);
            editCommand = new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE, 1);
            //this.addCommand(sendCommand);
            this.addCommand(editCommand);
        }
        
        //Display a non-autodismiss alert if the contact list is not ready
        if (!UIController.getContactsLoader().isLocked()) {
            setCallCommand();
        }
        
        this.setCommandListener(this);
        
    }
    
    public void enableCallCommand() {
        setCallCommand();
    }

    private void setCallCommand() {
        Address from = message.getFrom();
        // check in case of missing from like for messages in Sent Folder
        if (from != null) {
            removeCommand(callCommand);
            try {

                contactToCall = ((RMSContactManager) UIController.getContactManager()).searchContactFromMail(from.getEmail());

            } catch (MailException ex) {
                Log.error("Error searching contact from email address in ViewMessage setting CallCommand ");
                ex.printStackTrace();
            } catch (ContactManagerException ex) {
                Log.error("Error searching contact from email address in ViewMessage setting CallCommand ");
                ex.printStackTrace();
            }
            if (isValidForCall(contactToCall)) {
                addCommand(callCommand);
            }

        }
    }

    private void setMarkAsReadCommand() {

        removeCommand(markAsReadCommand);
        if (message.getFlags().isSet(MessageFlags.OPENED)) {
            markAsReadCommand = new Command(Localization.getMessages().UNREAD_COMMAND_LABEL, UIController.COMMAND_TYPE, 35);
        } else {
            markAsReadCommand = new Command(Localization.getMessages().READ_COMMAND_LABEL, UIController.COMMAND_TYPE, 35);
        }

        this.addCommand(markAsReadCommand);
    }

    private void setFollowUpCommand() {

        removeCommand(followUpCommand);
        if (message.getFlags().isSet(MessageFlags.FLAGGED)) {
            followUpCommand = new Command(Localization.getMessages().UNFLAG_COMMAND_LABEL, UIController.COMMAND_TYPE, 30);
        } else {
            followUpCommand = new Command(Localization.getMessages().FLAG_COMMAND_LABEL, UIController.COMMAND_TYPE, 30);
        }

        this.addCommand(followUpCommand);
    }

    /**
     *
     */
    public void commandAction(Command command, Displayable displayable) {

        if (command == UIController.backCommand) {

            try {
                if (!UIController.isSyncInProgress()) {
                    UIController.getInboxMessageList().resetTitle();
                }
//                UIController.getInboxMessageList().activate(
//                        UIController.getInboxMessageList().getActiveElementId());

                cleanup();    // FIXME
                //UIController.showInboxScreen();

                UIController.showBackScreen();

            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error("ViewMessage: " + ex);
            }
        } else if (command == UIController.nextCommand) {
            try {
                this.setTitle(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE);
                repaint();
                serviceRepaints();
                getNextMessage(NEXT_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        } else if (command == UIController.previousCommand) {
            try {
                this.setTitle(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE);
                repaint();
                serviceRepaints();
                getNextMessage(PREVIOUS_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        } else if (command == replyCommand) {
            UIController.replyMessage(message);
            cleanup();    // FIXME
        } else if (command == forwardCommand) {
            UIController.forwardMessage(message);
            cleanup();    //FIXME
            
            
            // TODO: when we'll go to the recipient list
            // from here, we'l need to remove the
            // cleanup call since if we wanna 
            // come back after the fw, we need to have 
            // the message or we'll throw a NPE
            
            
        } else if (command == deleteCommand) {
            deleteMessage();
        } else if (command == saveSenderCommand) {
            if (UIController.checkContactListFull()) {
                UIController.showErrorAlert(
                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
                return;
            }
            Contact contact;
            try {
                // resetting the message in contact list
                UIController.getContactList().setMessage(new Message());

                if (message.getFrom() != null) {
                    contact = new Contact(
                            message.getFrom().getName(),
                            message.getFrom().getEmail());
                    UIController.showAddContactScreen(
                            UIController.display.getCurrent(), contact);
                } else {
                    UIController.showAlert(Localization.getMessages().FROM_IS_NULL);
                }

            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "Mail Exception while adding Contact" + ex);
                UIController.showAddContactScreen(UIController.display.getCurrent());
                UIController.showErrorAlert(
                        Localization.getMessages().MESSAGE_VIEW_UNABLE_TO_CREATE_CONTACT);
            }

        } else if (command == linksCommand) {
            String content = message.getTextContent();
            UIController.showLinkList(UIController.display.getCurrent(), content);
        } else if (command == attachList) {
            UIController.showAttachmentsList(UIController.display.getCurrent(), message);
        } /*else if (command==numbersCommand ){
        UIController.showPhoneNumberList(UIController.display.getCurrent(), message.getTextContent());
        }*/ else if (command == recipientsCommand) {
            Log.info("showing recipients for message " + message.getSubject());
            UIController.showRecipientsList(UIController.display.getCurrent(), message);
        } else if (command == editCommand) {
            message.setMessageId(message.createUniqueMessageIDValue());
            UIController.updateComposeMessageScreen(message, "");
            UIController.showComposeMessageScreen(UIController.display.getCurrent(), UIController.getComposeMessageScreen().body);
        //XXX
        } else if (command == replyAllCommand) {
            UIController.replyAll(message);
            cleanup();    // FIXME
        } else if (command == markAsReadCommand) {
            UIController.changeReadFlag(message);
            setMarkAsReadCommand();
        } else if (command == followUpCommand) {
            UIController.changeFollowUpFlag(message);
            setFollowUpCommand();
            UIController.getInboxMessageList().activate(UIController.getInboxMessageList().getActiveElementId());
        } else if (command == fullMessageCommand) {
            this.removeCommand(fullMessageCommand);
            fullMsgPerformed = true;
            this.setTitle(Localization.getMessages().MVF_FULL_MESSAGE_LOADING);
            repaint();
            UIController.getFullMessage(message);
        } else if (command == callCommand) {
            UIController.phoneCall(contactToCall, this);
        } else if (command == UIController.blackberryExitCommand) {
            UIController.midlet.destroyApp(true);
        } else {
            Log.info("MessageViewForm: command not implemented");
        }
    }

    /*
    private void deleteMessage() {
    UIController.deleteMessage(message);
    cleanup();    // FIXME
    UIController.showInboxScreen();
    }
     */
    private void deleteMessage() {

        if (UIController.getConfig().getDeleteConfirmation()) {

            ModalPopup popup =
                    new ModalPopup(Localization.getMessages().POPUP_DELETE_MESSAGE_TITLE,
                    Localization.getMessages().POPUP_DELETE_MESSAGE_TEXT,
                    new DeletePopupAction(message));
            UIController.showModalPopup(popup, this);
        } else {
            UIController.deleteMessage(message);
            //UIController.showBackScreen();
        }
    }

    private void enterBody(int dir) {
        insideBody = true;
    }

    protected void keyRepeated(int keyCode) {
        keyPressed(keyCode);
    }

    private void drawScrollBar(Graphics g, int h) {

        int maxHeight = getHeight() - h;


        int barheight = (int) (((bodyItem.getLastLine() * 100 -
                bodyItem.getCurrentLine() * 100) / bodyItem.getNumLines()) * maxHeight) / 100;
        int barposition = (int) (h + ((100 * bodyItem.getCurrentLine() * maxHeight / bodyItem.getNumLines())) / 100);

        g.setColor(theme.getScrollbarBackgroundColor());
        g.fillRect(getWidth() - SCROLLBAR_WIDTH, h, SCROLLBAR_WIDTH, maxHeight);

        g.setColor(theme.getForegroundColor());
        g.drawRect(getWidth() - SCROLLBAR_WIDTH, h, SCROLLBAR_WIDTH, maxHeight);

        g.fillRect(getWidth() - SCROLLBAR_WIDTH, barposition, SCROLLBAR_WIDTH,
                barheight);
    }

    /**
     * FIXME: this is just a patch. Needs to find why the viewMessage is
     * not collected by the gc.
     */
    private void cleanup() {
        this.message = null;
        this.bodyItem = null;
        this.headerItem = null;
    }

    public void setMessage(Message fullmsg) {
        if (fullmsg == null) {
            Log.info("The server didn't send the full message, ");
            this.enableFullMessageCommand(true);
            return;
        }

        if (this.message == null) {
            // cleanup has been called
            Log.info("User closed the viewmessagescreen, discarding full message");
            return;

        }


        if (bodyItem == null || headerItem == null) {
            Log.info("bodyItem == null || headerItem == null ");
            return;
        }

        try {
            fullmsg.setParent(message.getParent());
            //Fix for message flags - see 2772 bug
            fullmsg.setFlags(message.getFlags());
            message = fullmsg;
            bodyItem.setMessage(message);
            headerItem.setMessage(message);

        } catch (MailException e) {
            this.setTitle(Localization.getMessages().MESSAGE_VIEW_FORM_TITLE);
            Log.error("Error getting VisibleName - " + e.getMessage());
            e.printStackTrace();
        } finally {
            this.removeCommand(fullMessageCommand);

            resetTitle();
            repaint();
        }
    }

    public void resetTitle() {
        if (message != null) {
            setTitle(generateTitle());
        }
    }

    /**
     * generate a title in the form of "Message x of Y"
     * @return the title
     */
    private String generateTitle() {
        MessageList fcml = UIController.getMessageList(message);
        return Localization.getMessages().getMessageXofY(
                fcml.getActiveElementId() + 1, fcml.getNumMessages());
    }

    private boolean isValidForCall(Contact contactToCall) {
        boolean result = false;
        if (contactToCall != null) {
            if (!StringUtil.isNullOrEmpty(contactToCall.getHomePhone()) || !StringUtil.isNullOrEmpty(contactToCall.getJobPhone()) || !StringUtil.isNullOrEmpty(contactToCall.getMobilePhone())) {
                result = true;
            }
        }
        return result;
    }

//------------------------------BODYITEM CLASS ---------------------------//
    /**
     * This class handle the visualization of the body of the message.
     */
    class BodyItem {

        private int currentLine;
        private int maxLineIndex;
        private int fontHeight;
        private Vector textStringsVector;
        private String[] textStrings;
        private int bodyLastWidth = -1;
        private int screenHeight;
        private int lastline;

        public BodyItem() {
            fontHeight = bodyFont.getHeight();
            screenHeight = getHeight();
        }

        public void traverse(int dir) {
            /*
            for future implementation of internal navigation
            if( dir == Canvas.RIGHT) {
            toNextMessage();
            return true;
            } else if (dir == Canvas.LEFT) {
            toPreviousMessage();
            return true;
            }
             */
            if (insideBody) {
                if (dir == Canvas.DOWN) {
                    scroll(Canvas.DOWN);
                    if (isAtBottom()) {

                        currentLine = Math.max(maxLineIndex + 2 -
                                screenHeight / fontHeight, 0);
                        //visRect_inout[1] = calculateY(viewportHeight);
                        exitBody(dir);
                    }
                } else if (dir == Canvas.UP) {
                    scroll(Canvas.UP);
                    if (isAtTop()) {
                        currentLine = 0;
                        exitBody(dir);
                    }
                }
            }
        }

        public void setMessage(Message fullmsg) {
            //we add a \n to the messagebody for nicer visualization
            String content = fullmsg.getTextContent();
            textStrings = UiUtils.getStringArray(content + " \r\n", bodyLastWidth, bodyFont);
            // Flush the content to save memory
            maxLineIndex = Math.max(textStrings.length - 1, 1);
        //       Log.debug (this, "new max line index " + maxLineIndex);

        }

        private void scroll(int dir) {
            if (dir == Canvas.DOWN) {
                currentLine += SCROLL_AMOUNT;
            } else if (dir == Canvas.UP) {
                currentLine -= SCROLL_AMOUNT;
            }
        // Log.info ("Currentline: " + currentLine);
        }

        /**
         * draw the element with given width starting at given height
         *
         *@return the height to start painting the next item
         */
        public int draw(Graphics g, int w, int h) {
            //  Log.debug(this, "initializing body vectors");
            initializeBodyStringVectors(w - 3 * MARGIN);
            //   Log.debug(this, " body vectors initialized");

            //  drawer.drawBackground(g,w,h,false);

            //we draw the lines
            int i;
            for (i = currentLine; i <= currentLine + screenHeight / fontHeight; i++) {
                if (i < textStrings.length) {
                    drawer.drawString(textStrings[i], g, MARGIN, h, false);
                    h += fontHeight;

                } else {
                    break;
                }
            }
            lastline = i;

            return h;
        }

        private void initializeBodyStringVectors(int w) {
            if (bodyLastWidth != w) {

                //we add a \n to have nicer visualization
                textStrings = UiUtils.getStringArray(message.getTextContent() + " \n \n", w,
                        drawer.getGraphicalTheme().getDefaultFont());

                bodyLastWidth = w;
                maxLineIndex = Math.max(textStrings.length - 1, 1);
            }
        }

        private boolean isAtBottom() {
            return (currentLine - 2 + screenHeight / fontHeight >= maxLineIndex);
        }

        private boolean isAtTop() {
            return (currentLine <= 0);
        }

        protected int getCurrentLine() {
            return currentLine;
        }

        protected int getLastLine() {
            return lastline;
        }

        protected int getNumLines() {
            return maxLineIndex;
        }
    }

    //------------------------------- HEADERITEM CLASS -----------------------//
    /**
     * This class handle the visualization of the body of the message.
     */
    class HeaderItem {

        private int headerLastWidth = -1;
        private boolean expanded;
        /**
         * message fields
         */
        private String from = "";
        private String subject = "";
        private StringBuffer to = new StringBuffer();
        private String date = "";
        private StringBuffer cc = new StringBuffer();
        private StringBuffer bcc = new StringBuffer();
        /**
         * String constants
         */
        private final String FROM = Localization.getMessages().MVHI_FROM_LABEL;
        private final String SUBJECT = Localization.getMessages().MVHI_SUBJ_LABEL;
        private final String TO = Localization.getMessages().TO_LABEL;
        private final String DATE = Localization.getMessages().MVHI_ON_LABEL;
        private final String CC = Localization.getMessages().CC_LABEL;
        private final String BCC = Localization.getMessages().BCC_LABEL;
        /**
         * vector to store lines
         */
        private String[] fromStrings;
        private String[] subjectStrings;
        private String toStrings;
        private String[] dateStrings;
        private String ccStrings;
        private String bccStrings;
        private final int BORDER = 2;
        private Font fromFont = theme.getSmallFont();
        private Font toFont = theme.getSmallFont();
        private Font subjectFont = theme.getSmallFont();
        private Font hiddenFieldsFont = theme.getSmallFont();
        private Font visibleFieldsFont = theme.getSmallFont();
        private int subjectColor = theme.getHighlightedForegroundColor();
        private int hiddenFieldsColor = theme.getHighlightedForegroundColor();
        private int visibleFieldsColor = theme.getHighlightedForegroundColor();

        public HeaderItem() {
            //filling fields..
            setMessage(message);

        }

        private void setFonts() {
            /*  if (UiUtils.isIncoming(message) ) {
            fromFont = theme.getDefaultFont();
            toFont = theme.getSmallFont();
            } else {
            fromFont = theme.getSmallFont();
            toFont = theme.getDefaultFont();
            }
             */
            subjectFont = theme.getSmallBoldFont();
            subjectColor = theme.getHighlightedForegroundColor();
        }

        private void setMessage(Message msg) {
            try {
                if (message.getFrom() != null) {
                    from = message.getFrom().getVisibleName();
                } else {
                    from = "";
                }
            } catch (MailException ex) {
                Log.error(this, "error getting from field");
                ex.printStackTrace();
            }
            to = new StringBuffer();
            cc = new StringBuffer();
            bcc = new StringBuffer();
            try {
                Address[] toAddr = message.getTo();
                int count = toAddr == null ? 0 : toAddr.length;
                for (int i = 0; i < count; i++) {
                    to.append((i == 0) ? "" : ", ");
                    to.append(toAddr[i].getVisibleName().trim());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching to addresses");
            }

            try {
                Address[] ccAddr = message.getCc();
                int count = ccAddr == null ? 0 : ccAddr.length;
                for (int i = 0; i < count; i++) {
                    cc.append((i == 0) ? "" : ", ");
                    cc.append(ccAddr[i].getVisibleName().trim());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching cc addresses");
            }

            try {
                Address[] bccAddr = message.getBcc();
                int count = bccAddr == null ? 0 : bccAddr.length;
                for (int i = 0; i < count; i++) {
                    bcc.append((i == 0) ? "" : ", ");
                    bcc.append(bccAddr[i].getVisibleName().trim());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching Bcc addresses");
            }

            Date dateObj = message.getSentDate();
            // Calendar calendar = Calendar.getInstance();
            // calendar.setTime(dateObj);
            // calendar.setTimeZone(TimeZone.getDefault());

            // this is only for visualization purposes
            //TODO: check if the variables in formatLocalTime are really used

            //date = MailDateFormatter.formatLocalTime(calendar.getTime());
            date = MailDateFormatter.formatLocalTime(dateObj);
            subject = message.getSubject();

            setFonts();

        }

        /**
         *a traverse-like method that updates the state of this item
         */
        public void traverse(int dir) {

            // Log.debug("traverse header Item, insideheader = " + insideHeader);
            if (insideHeader) {
                if ((dir == Canvas.UP)) {
                    //  Log.debug("traverse up");
                    if (!expanded) {
                        expand();
                    }
                //else expand();
                } else if ((dir == Canvas.DOWN)) {
                    //   Log.debug("traverse down");
                    if (expanded) {
                        shrink();
                    } else {
                        exitHeader(dir);
                    }
                }
            }
        }

        private void shrink() {
            //  Log.debug("shrink()");
            expanded = false;
        }

        private void expand() {
            //   Log.debug("expand()");
            expanded = true;
        }

        /**
         * @return the estimated height for this item, used to draw the
         * background etc.
         */
        protected int calculateHeight(int width) {

            if (!showHeader) {
                return (0);
            }

            //TODO: check why sometimes passed width is 0
            if (width == -1 || width == 0) {
                initializeHeaderStringVectors(width - 4 * BORDER);
            } else {
                initializeHeaderStringVectors(width - 4 * BORDER);
            }

            int h = subjectFont.getHeight() * subjectStrings.length;
            if (UiUtils.isIncoming(message)) {
                h += visibleFieldsFont.getHeight() * fromStrings.length;
            } else {
                if (toStrings != null) {
                    h += visibleFieldsFont.getHeight();
                }
            }

            if (expanded) {
                if (UiUtils.isIncoming(message)) {
                    h += hiddenFieldsFont.getHeight();
                } else {
                    h += hiddenFieldsFont.getHeight() * fromStrings.length;
                }

                h += hiddenFieldsFont.getHeight() * dateStrings.length;
                if (ccStrings != null) {
                    h += hiddenFieldsFont.getHeight();
                }
                if (bccStrings != null) {
                    h += hiddenFieldsFont.getHeight();
                }
            }
            h += 4 * BORDER;
            return h;
        }

        private void initializeHeaderStringVectors(int w) {
            if (headerLastWidth != w) {
                int width = w - 4 * BORDER;
                if (from.equals("")) {
                    fromStrings = new String[0];
                } else {
                    fromStrings = UiUtils.getStringArray(FROM + " " + from, width, fromFont);
                }
                subjectStrings = UiUtils.getStringArray(SUBJECT + " " + subject, width, subjectFont);
                if (to.length() != 0) {
                    int wid = width;
                    if (message.isMultipart()) {
                        wid -= Theme.getCLIP_IMG().getWidth();
                    }

                    toStrings = UiUtils.cutString(TO + " " + to, wid, toFont);//getStringArray(TO+" "+to, width,toFont);
                } /*else {
                toStrings= new String();
                }*/

                dateStrings = UiUtils.getStringArray(DATE + " " + date, width, hiddenFieldsFont);
                if (cc.length() != 0) {
                    ccStrings = UiUtils.cutString(CC + " " + cc, width, hiddenFieldsFont);//UiUtils.getStringArray(CC+" " +cc, width,hiddenFieldsFont);
                } /*else {
                ccStrings=new String();
                }*/
                if (bcc.length() != 0) {
                    bccStrings = UiUtils.cutString(BCC + " " + bcc, width, hiddenFieldsFont);//getStringArray(CC+" " +cc, width,hiddenFieldsFont);
                } /*else {
                bccStrings=new String();
                }*/
                headerLastWidth = w;
            }

        }

        /**
         * paint function. it draws the strings ecc.
         *h it's the current graphics height to print at,
         *it's modified by this function to return the next y where content
         *can be drawn.
         *@return the h + the height drawn by the item
         */
        protected int draw(Graphics graphics, int w, int h) {
            //graphics.getFont().
            //   int oldCol=graphics.getColor();

            //   Log.debug("drawing headerItem");


            if (!showHeader) {
                return h;
            }
            int inith = h;
            initializeHeaderStringVectors(w - 2 * BORDER);
            int height = calculateHeight(w);


            //we need to translate the graphics, because the drawer.drawbackground
            //method starts at 0,0
            //graphics.translate(0, h);
            drawer.drawBackground(graphics, 0, h, w, height, true);
            //graphics.translate(-graphics.getTranslateX(), -graphics.getTranslateY());

            /*
            graphics.setColor(theme.getHighlightedBackgroundColor());
            //TODO: use drawer and optimize!
            graphics.fillRoundRect(BORDER,h,w-2*BORDER,height,10,10);
            graphics.setColor(theme.getHighlightedBorderColor());
            graphics.drawRoundRect(BORDER,h,w-2*BORDER,height,10,10);
             */

            h += 2 * BORDER;

            //we set the font to draw the "from: address" line and we draw it
            //graphics.getDisplayColor()

            //if expanded we draw to, cc, bbc & date fields
            if (expanded) {
                if (UiUtils.isIncoming(message)) {
                    h = paintToField(graphics, h);
                } else {
                    h = paintFromField(graphics, h);
                }
                h = paintCcField(graphics, h);
                h = paintBccField(graphics, h);
                h = paintDateField(graphics, h);
            }

            if (UiUtils.isIncoming(message)) {
                h = paintFromField(graphics, h);
            } else {
                h = paintToField(graphics, h);
            }
            h = paintSubjectField(graphics, h);


            graphics.setColor(theme.getHighlightedForegroundColor());
            if (!expanded) {
                //a small arrow up to let the user know we have something hidden that
                // can be discovered pressing the up key
                graphics.fillTriangle(w - 9 - BORDER, inith + 8 + BORDER, w - 6 - BORDER, inith + 1 + BORDER, w - 2 - BORDER, inith + 8 + BORDER);
            } else {
                //small arrow down
                graphics.fillTriangle(w - 9 - BORDER, inith + 1 + BORDER, w - 6 - BORDER, inith + 8 + BORDER, w - 2 - BORDER, inith + 1 + BORDER);
            }
            return height + inith;
        }

        private int paintSubjectField(final Graphics graphics, int h) {
            //we set the font to draw the subject line
            graphics.setFont(subjectFont);
            graphics.setColor(subjectColor);
            //we draw the lines
            for (int i = 0; i < subjectStrings.length; i++) {
                graphics.drawString(subjectStrings[i], 2 * BORDER, h, graphics.TOP | graphics.LEFT);
                h += subjectFont.getHeight();
            }
            return h;
        }

        private int paintDateField(final Graphics graphics, int h) {
            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);
            for (int i = 0; i < dateStrings.length; i++) {
                graphics.drawString(dateStrings[i], 2 * BORDER, h, graphics.TOP | graphics.LEFT);
                h += hiddenFieldsFont.getHeight();
            }
            return h;
        }

        private int paintBccField(final Graphics graphics, int h) {
            if (bccStrings == null) {
                return h;
            }
            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);

            graphics.drawString(bccStrings, 2 * BORDER, h, graphics.TOP | graphics.LEFT);
            h += hiddenFieldsFont.getHeight();

            return h;

        }

        private int paintCcField(final Graphics graphics, int h) {
            if (ccStrings == null) {
                return h;
            }

            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);

            graphics.drawString(ccStrings, 2 * BORDER, h, graphics.TOP | graphics.LEFT);
            h += hiddenFieldsFont.getHeight();

            return h;
        }

        private int paintFromField(final Graphics graphics, int h) {
            int width = 0;
            graphics.setColor(theme.getHighlightedForegroundColor());
            graphics.setFont(fromFont);
            for (int i = 0; i < fromStrings.length; i++) {
                //drawer.drawString(fromStrings[i], graphics, 2*BORDER,h,true);
                graphics.drawString(fromStrings[i], 2 * MARGIN, h, graphics.TOP | graphics.LEFT);
                h += fromFont.getHeight();
                width = fromFont.stringWidth(fromStrings[i]);
            }
            if (UiUtils.isIncoming(message) && message.isMultipart()) {
                graphics.drawImage(Theme.getCLIP_IMG(), width + 3 * MARGIN,
                        h - Theme.getCLIP_IMG().getWidth() - MARGIN, Graphics.TOP | Graphics.LEFT);
            }
            return h;
        }

        private int paintToField(final Graphics graphics, int h) {
            if (toStrings == null) {
                return h;
            }
            graphics.setColor(theme.getHighlightedForegroundColor());
            graphics.setFont(toFont);

            graphics.drawString(toStrings, 2 * MARGIN, h, graphics.TOP | graphics.LEFT);
            h += toFont.getHeight();
            if (UiUtils.isSent(message) && message.isMultipart()) {

                int width = toFont.stringWidth(toStrings);
                graphics.drawImage(Theme.getCLIP_IMG(), width + 3 * MARGIN,
                        h - Theme.getCLIP_IMG().getWidth() - MARGIN, Graphics.TOP | Graphics.LEFT);
            }
            return h;

        }
    }
}
